<?php
ini_set('display_errors', 1);
ini_set('log_errors', 1);
ini_set('error_log', dirname(__FILE__) . '/'.basename(__FILE__).'.info_log.txt');
error_reporting(E_ALL & ~E_NOTICE);
ini_set('memory_limit', '400M');
$_SERVER['SCRIPT_NAME'] = "/" . $argv[0];
//allows to pass the domain to wich apply this script
isset($argv[1]) ? $_SERVER['HTTP_HOST'] = $argv[1] : FALSE;
include_once './includes/bootstrap.inc';
drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);

function jsonFormat($json) {
    if (!is_string($json)) {
      if (phpversion() && phpversion() >= 5.4) {
        return json_encode($json, JSON_PRETTY_PRINT);
      }
      $json = json_encode($json);
    }
    $result = '';
    $pos = 0; // indentation level
    $strLen = strlen($json);
    $indentStr = "\t";
    $newLine = "\n";
    $prevChar = '';
    $outOfQuotes = true;

    for ($i = 0; $i < $strLen; $i++) {
      // Grab the next character in the string
      $char = substr($json, $i, 1);

      // Are we inside a quoted string?
      if ($char == '"' && $prevChar != '\\') {
        $outOfQuotes = !$outOfQuotes;
      }
      // If this character is the end of an element,
      // output a new line and indent the next line
      else if (($char == '}' || $char == ']') && $outOfQuotes) {
        $result .= $newLine;
        $pos--;
        for ($j = 0; $j < $pos; $j++) {
          $result .= $indentStr;
        }
      }
      // eat all non-essential whitespace in the input as we do our own here and it would only mess up our process
      else if ($outOfQuotes && false !== strpos(" \t\r\n", $char)) {
        continue;
      }

      // Add the character to the result string
      $result .= $char;
      // always add a space after a field colon:
      if ($char == ':' && $outOfQuotes) {
        $result .= ' ';
      }

      // If the last character was the beginning of an element,
      // output a new line and indent the next line
      if (($char == ',' || $char == '{' || $char == '[') && $outOfQuotes) {
        $result .= $newLine;
        if ($char == '{' || $char == '[') {
          $pos++;
        }
        for ($j = 0; $j < $pos; $j++) {
          $result .= $indentStr;
        }
      }
      $prevChar = $char;
    }
    return $result;
}
function getNodes($node, $types) {
  $node->repository_tree = getRepositoryNodes($node->nid,$types); 
  return jsonFormat(json_encode($node));
}
function getRepositoryNodes($nid, $node_types) {
  $group_nodes = og_group_child_nids($nid);
  addProjectRelease($nid,$group_nodes);
  $nodes = array();
  foreach($group_nodes as $key => $value) {
    $in = "('" . implode("','", $node_types) . "')";
    $sql = "SELECT type FROM node WHERE type IN $in AND nid=%d";
    $type = db_result(db_query($sql,$value));
    if ("" != $type) {
      $root_node = node_load($value);
      $node = getReferencedNode($root_node);
      $nodes[$type][] = $node;
    } 
  }
  return $nodes;
}
function addProjectRelease($nid, &$group_nodes){
  $node = node_load($nid);
  if ('project_project' == $node->type and 'software' == $node->group_type) {
    $sql = "SELECT 
            node.nid, node.title
            FROM node node
            LEFT JOIN project_release_nodes project_release_nodes ON node.nid = project_release_nodes.nid
            LEFT JOIN node node_project_release_nodes ON project_release_nodes.pid = node_project_release_nodes.nid
            INNER JOIN project_projects node_project_release_nodes__project_projects ON node_project_release_nodes.nid = node_project_release_nodes__project_projects.nid
            WHERE (node.type in ('project_release'))
            AND (node_project_release_nodes__project_projects.uri = '%s')";    
    $result = db_query($sql,$node->project['uri']);
    while ($row = db_fetch_object($result)) {
      $group_nodes[] = $row->nid;
    }
  }
}
function getReferencedNode($node) {
  $fields = content_fields(NULL, $node->type);
  $fields = nodeReferenceTypes($fields);
  foreach($node as $key => $value) {
    if (in_array($key,$fields)) {
      foreach($value as $sub_key => $sub_value) {
        $node->{$key}[$sub_key]['node'] = getReferencedNode(node_load($value[0]['nid']));
      }
    }
  }
  return $node;
}
function nodeReferenceTypes($fields) {
  $nodeReference = array();
  foreach($fields as $key => $value) {
    if ("nodereference" == $value['type']) {
      $nodeReference[] = $key;
    }
  }
  return $nodeReference;
}
function getRepositories() {
  if (!is_dir(dirname(__FILE__) . "/exported_repositories")) {
    mkdir(dirname(__FILE__) . "/exported_repositories");
  }
  $num_repositories = db_result(db_query("SELECT COUNT(nid) FROM node WHERE type='repository'"));
  $result = db_query("SELECT nid FROM node WHERE type='repository'");
  $i=1;
  while ($row = db_fetch_object($result)) {
    echo "[$i of $num_repositories] repository " . $row->nid . "\n";
    error_log("[".$i." of ".$num_repositories."] repository " . $row->nid . "\n");
    $i++;
    $node = getNodes(node_load($row->nid), array("adms_software_project",'asset_release'));
    if(false === file_put_contents(dirname(__FILE__) . "/exported_repositories/" . $row->nid . ".rdf", $node)){
      error_log("--Err: file: " . dirname(__FILE__) . "/exported_repositories/" . $row->nid . ".rdf \n");
    }
  }
}
function getProjects() {
  if (!is_dir(dirname(__FILE__) . "/exported_projects")) {
    mkdir(dirname(__FILE__) . "/exported_projects");
  }
  $num_projects = db_result(db_query("SELECT COUNT(nid) FROM node WHERE type='project_project'"));
  $result = db_query("SELECT nid FROM node WHERE type='project_project'");
  $i=1;
  while ($row = db_fetch_object($result)) {
    echo "[$i of $num_projects] project " . $row->nid . "\n";
    error_log("[".$i." of " . $num_projects."] repository " . $row->nid . "\n");

    $i++;
    $node = getNodes(node_load($row->nid), array('asset_release','project_release'));
    if(false === file_put_contents(dirname(__FILE__) . "/exported_projects/" . $row->nid . ".rdf", $node)){
      error_log("--Err: file: " . dirname(__FILE__) . "/exported_projects/" . $row->nid . ".rdf \n");
    }
  } 
}
getRepositories();
getProjects();
