<?php

/**
 * Handler to migrate blogs from Epractice website
 * @author antartem
 *
 */
class EpracticeBlogMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));
		$json_file =  MIGRATION_FILES_JSON.'/blog.json';
		$this->map = new MigrateSQLMap($this->machineName,
			array(
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				),
			),
			MigrateDestinationNode::getKeySchema()
		);

		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';

		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());

		$this->destination = new MigrateDestinationNode('blog');


		$this->addFieldMapping('uid', 'uid')->defaultValue(0)
			->sourceMigration('EpracticeUser');

		$this->addSimpleMappings(array('title', 'body', 'created','status', 'format'));

		$this->addFieldMapping ( 'language', 'language' )->callbacks ( array (
		  $this,
		  'getLowerCase'
		) );

		$this->addFieldMapping('taxonomy', 'taxonomy')
  		->sourceMigration('EpracticeTaxonomy')
  		->arguments(array('source_type' => 'tid'));
	}


	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
			'nid' => 'Node id',
			'uid' => 'User id',
			'language' => '',
			'format' => '',
			'created' => '',
			'changed' => '',
			'status' => '',
			'title' => '',
  			'body' => '',
  			'teaser' => '',
  			'og_groups' => '',
		    'field_blog_tags' => '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_taxonomy_to_array($row);
		_update_textarea_links($row);
		return TRUE;
	}

	/**
	 * Change multiple array fields and rise it one level
	 * Import related files
	 * @param stdClass $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		_epractice_og_content($node, null);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
		
		
	}

	/**
	 *
	 * @param string $value
	 * @return string
	 */
	function getLowerCase($value) {
	 return strtolower ( $value );
	}
	
	/**
     * Change changed date
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
		_update_changed_date($node->nid,$row->changed);
	}

}