<?php

/**
 * Handler to migrate communities from Epractice website
 * @author antartem
 *
 */
class EpracticeCaseMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {

		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));

		$json_file =  MIGRATION_FILES_JSON.'/cases.json';
		$http_options = array();
		$this->map = new MigrateSQLMap($this->machineName,
			array(
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				),
			),
			MigrateDestinationNode::getKeySchema()
		);

		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';
		
		/*
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, 'nid'),
				new EpracticeItemJSON($json_file, $http_options, 'nid'),
				$this->fields()
		);
		*/
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());
		
		$this->destination = new MigrateDestinationNode('case_epractice');
		$this->addFieldMapping('uid', 'uid')->defaultValue(0)
			->sourceMigration('EpracticeUser');
		$this->addSimpleMappings(array('title', 'created','format','status','language'));
		$this->addFieldMapping('body', 'field_cases_abstract');
		$this->addFieldMapping('field_acronym', 'field_cases_acronym');
		$this->addFieldMapping('field_presentation_url', 'field_cases_web');
		$this->addFieldMapping('field_website_url', 'field_cases_url');
		$this->addFieldMapping('field_city_region', 'field_cases_city');

		$this->addFieldMapping('field_policy_context', 'field_cases_policy');
		$this->addFieldMapping('field_desc_target_users_groups', 'field_cases_target_description');
		$this->addFieldMapping('field_desc_implementation', 'field_cases_implementation_way');
		$this->addFieldMapping('field_funding_source', 'field_cases_funding');
		$this->addFieldMapping('field_tech_solution', 'field_cases_technology_solution');
		$this->addFieldMapping('field_main_results', 'field_cases_impact');
		$this->addFieldMapping('field_track_record_sharing', 'field_cases_sharing');
		$this->addFieldMapping('field_lessons_learnt', 'field_cases_lessons');
		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
	}


	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'created' => '',
				'changed' => '',
				'format' => '',
				'status' => '',
				'title' => '',
				'field_cases_abstract' => '',
				'field_cases_acronym' => '',
				'field_cases_web' => '',
				'field_cases_logo' => '',
				'field_cases_url' => '',
				'field_cases_city' => '',
				'field_cases_date' => '',
				'field_cases_period' => '',
				'field_cases_policy' => '',
				'field_cases_target_description' => '',
				'field_cases_implementation_way' => '',
				'field_cases_technology_solution' => '',
				'field_cases_type_initiative' => '',
				'field_cases_overall' => '',
				'field_cases_technology' => '',
				'field_cases_funding' => '',
				'field_cases_impact' => '',
				'field_cases_sharing' => '',
				'field_cases_lessons' => '',
				'field_cases_domain' => '',
				'og_groups'=> '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_og_groups_to_array($row);
		_cast_taxonomy_to_array($row);
		_update_textarea_links($row);
		return TRUE;
	}

	/**
	 * Change multiple array fields and rise it one level
	 * Import related files
	 * @param unknown $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		$node->type = "case_epractice";
		_cast_node_object_to_array($node);
		_epractice_file_import($row->field_cases_logo, $node->field_case_logo, $node->uid);
		_cast_import_taxonomy_field($row->field_cases_technology, $node->field_cases_technology);
		_cast_import_taxonomy_field($row->field_cases_overall, $node->field_cases_overall);
		_cast_import_taxonomy_field($row->field_cases_sector, $node->field_case_sector);
		_format_date_field($row->field_cases_period[0], $node->field_start_end_date);
		_format_date_field($row->field_cases_date[0], $node->field_operational_date);

		_epractice_og_content($node,$row->field_cases_domain);
		_add_urls($row->field_cases_web[0], $node->field_website_url);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}		
		if(isset($node->field_case_sector)&&is_array($node->field_case_sector)&&isset($node->taxonomy)&&is_array($node->taxonomy)){
		  foreach($node->field_case_sector as $sector){
		    foreach($node->taxonomy as $key=>$term){
		      if($sector['value']==$term){
			    unset($node->taxonomy[$key]);
			  }
		    }
		  }
		}
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
        if($sid = _get_workflow_state($row->_workflow, $node->type)){
          workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
        }
		_highlight_content($node, $row->field_cases_domain);
		foreach($row->og_groups as $gid){
		  $results = db_query("SELECT destid1 FROM migrate_map_epracticecommunity WHERE sourceid1=%d",$gid);
		  if($dest = db_fetch_array($results)){
	        _highlight_content($node, $dest['destid1']);
		  }
	    }
		_update_changed_date($node->nid,$row->changed);
	}
}