<?php

/**
 * Handler to migrate user from Epractice website
 * @author antartem
 *
 */
class EpracticeEdocumentMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		parent::__construct ( MigrateGroup::getInstance ( 'isa_epractice_migration' ) );

		$json_file =  MIGRATION_FILES_JSON.'/edocument.json';

		// Human-friendly description of your migration process. Be as detailed as you
		// like.
		$this->description = t ( 'Migrate eDocument nodes to Joinup' );

		// no options, or you could specify some
		$http_options = array ();

		$this->map = new MigrateSQLMap ( $this->machineName, array (
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				)
		), MigrateDestinationTerm::getKeySchema () );


		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';

		/*
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, 'nid'),
				new EpracticeItemJSON($json_file, $http_options, 'nid'),
				$this->fields()
		);*/
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());

		$this->destination = new MigrateDestinationNode('document');
		$this->addFieldMapping('uid', 'uid')
			->sourceMigration('EpracticeUser');
		$this->addFieldMapping('language', 'language');
		$this->addFieldMapping('created', 'created');
		$this->addFieldMapping('status', 'status');
		$this->addFieldMapping('title', 'title');
		$this->addFieldMapping('body', 'body');
		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'created' => '',
				'status' => '',
				'title' => '',
				'body' => '',
				'teaser' => '',
				'field_library_publish'=> '',
				'field_library_publisher'=> '',
				'field_library_isbn'=> '',
				'field_library_license'=> '',
				'field_library_additional_doc'=> '',
				'field_library_url'=> '',
				'field_library_email'=> '',
				'field_library_tags'=> '',
				'field_library_domain'=> ''
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_og_groups_to_array($row);
		_cast_taxonomy_to_array($row);
 		_update_textarea_links($row);
		return TRUE;
	}

	/**
	 * Modify row and node objects
	 * @param unknown $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		_epractice_og_content($node, NULL);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
      if($sid = _get_workflow_state($row->_workflow, $node->type)){
        workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
      }
	  _highlight_content($node, $row->field_library_domain);
    }
}
