<?php

/**
 * Handler to migrate user from Epractice website
 * @author antartem
 *
 */
class EpracticeElibraryMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		parent::__construct ( MigrateGroup::getInstance ( 'isa_epractice_migration' ) );

		$json_file =  MIGRATION_FILES_JSON.'/library.json';

		// Human-friendly description of your migration process. Be as detailed as you
		// like.
		$this->description = t ( 'Migrate eLibrary nodes to Joinup' );

		// no options, or you could specify some
		$http_options = array ();

		$this->map = new MigrateSQLMap ( $this->machineName, array (
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				)
		), MigrateDestinationTerm::getKeySchema () );


		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';

		/*
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, 'nid'),
				new EpracticeItemJSON($json_file, $http_options, 'nid'),
				$this->fields()
		);
		*/
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());
		
		$this->destination = new MigrateDestinationNode('document');
		$this->addFieldMapping('uid', 'uid')
			->sourceMigration('EpracticeUser')
			->defaultValue(0);
		$this->addFieldMapping('language', 'language');
		$this->addFieldMapping('created', 'created');
		$this->addFieldMapping('format', 'format');
		$this->addFieldMapping('status', 'status');
		$this->addFieldMapping('title', 'title');
		$this->addFieldMapping('body', 'body');
		$this->addFieldMapping('field_publisher', 'field_library_publisher');
		$this->addFieldMapping('field_isbn', 'field_library_isbn');
		$this->addFieldMapping('field_description_of_license', 'field_library_license_desc');
		$this->addFieldMapping('field_additional_doc_desc', 'field_library_complete_title');
		$this->addFieldMapping('field_email', 'field_library_email');
		$this->addFieldMapping('field_document_author', 'field_library_author');
		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'format' => '',
				'created' => '',
				'changed' => '',
				'status' => '',
				'title' => '',
				'body' => '',
				'teaser' => '',
				'og_groups' => '',
				'field_library_publish'=> '',
				'field_library_publisher'=> '',
				'field_library_isbn'=> '',
				'field_library_license'=> '',
				'field_library_additional_doc'=> '',
				'field_library_url'=> '',
				'field_library_email'=> '',
				'field_library_tags'=> '',
				'field_library_domain'=> '',
				'og_groups'=> '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_og_groups_to_array($row);
		_cast_taxonomy_to_array($row);
 		_update_textarea_links($row);
		return TRUE;
	}

	/**
	 * Modify row and node objects
	 * @param unknown $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		_epractice_file_import($row->field_library_additional_doc, $node->field_additional_doc_file, $node->uid);
		_epractice_og_content($node, $row->field_library_domain);
		_add_licence($node,$row->field_library_license[0]);
		_format_date_field($row->field_library_publish[0], $node->field_publication_date);
		_add_urls($row->field_library_url[0], $node->field_original_url);
		_cast_import_taxonomy_field($row->field_library_languages, $node->field_languages_fields, TRUE);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
      if($sid = _get_workflow_state($row->_workflow, $node->type)){
        workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
      }
	  _highlight_content($node, $row->field_library_domain);
	  foreach($row->og_groups as $gid){
		$results = db_query("SELECT destid1 FROM migrate_map_epracticecommunity WHERE sourceid1=%d",$gid);
		if($destid = db_fetch_array($results)){
	      _highlight_content($node, $destid['destid1']);
		}
	  }
	  _update_changed_date($node->nid,$row->changed);
    }
}
