<?php

/**
 * Handler to migrate events from Epractice website
 * @author mfontsan
 *
 */
class EpracticeEventsMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		//
		parent::__construct ( MigrateGroup::getInstance ( 'isa_epractice_migration' ) );

		$json_file = MIGRATION_FILES_JSON.'/event.json';
		// /die($json_file);
		// no options, or you could specify some
		$http_options = array ();
		// map for the migration
		$this->map = new MigrateSQLMap ( $this->machineName, array (
				'nid' => array (
						'type' => 'int',
						'unsigned' => TRUE,
						'not null' => TRUE
				)
		), MigrateDestinationNode::getKeySchema () );

		$this->dependencies [] = 'EpracticeTaxonomy';
		$this->dependencies [] = 'EpracticeUser';

		//$this->source = new MigrateSourceList ( new EpracticeListJSON ( $json_file, $http_options, 'nid' ), new EpracticeItemJSON ( $json_file, $http_options, 'nid' ), $this->fields () );

		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());

		$this->destination = new MigrateDestinationNode ( 'event' );

		$this->addFieldMapping ( 'uid', 'uid' )->defaultValue ( 0 )->sourceMigration ( 'EpracticeUser' );
		$this->addSimpleMappings ( array (
				'status',
				'body',
				'created',
				'format',
				'title'
		) );
		$this->addFieldMapping ( 'language', 'language' )->callbacks ( array (
				$this,
				'getLowerCase'
		) );
		$this->addFieldMapping ( 'field_event_organiser', 'field_event_organizer' )->callbacks ( array (
				$this,
				'getStringFromArray'
		) );
		$this->addFieldMapping ( 'field_event_exp_participants', 'field_event_participants' )->callbacks ( array (
				$this,
				'getStringFromArray'
		) );
		$this->addFieldMapping ( 'field_event_city', 'field_event_city_location' )->callbacks ( array (
				$this,
				'getStringFromArray'
		) );
		$this->addFieldMapping ( 'field_event_venue', 'field_event_venue' )->callbacks ( array (
				$this,
				'getStringFromArray'
		) );
		$this->addFieldMapping ( 'field_event_contact_email', 'field_event_email' )->callbacks ( array (
				$this,
				'getStringFromArray'
		) );
		$this->addFieldMapping ( 'teaser', 'field_event_summary' );
		$this->addFieldMapping ( 'field_event_fees_description', 'field_event_fees' )->callbacks ( array (
				$this,
				'getStringFromArray'
		) );
		$this->addFieldMapping ( 'field_event_state', 'field_event_state' )->callbacks ( array (
				$this,
				'getStringFromArray'
		) );

		$this->addFieldMapping ( 'taxonomy', 'taxonomy' )->sourceMigration ( 'EpracticeTaxonomy' )->arguments ( array (
				'source_type' => 'tid'
		) );
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 *
	 * @return array
	 */
	function fields() {
		return array (
				// 'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'format' =>'',
				'created' => '',
				'changed' => '',
				'status' => '',
				'title' => '',
				'body' => '',
				'teaser' => '',
				'field_event_organizer' => '',
				'field_events_organisation_type' => '',
				'field_event_start' => '',
				'field_event_end' => '',
				'field_event_domain_topics' => '',
				'field_event_topic' => '',
				'field_event_language' => '',
				'field_event_other_languages' => '',
				'field_event_participants' => '',
				'field_event_free' => '',
				'field_event_open' => '',
				'field_event_scope' => '',
				'field_event_country' => '',
				'field_event_city_location' => '',
				'field_event_venue' => '',
				'field_event_website' => '',
				'field_event_email' => '',
				'field_event_documentation' => '',
				'field_event_tags' => '',
				'field_event_state' => '',
				'field_event_summary' => '',
				'field_event_languages' => '',
				'field_event_fees' => '',
				'field_event_publish' => '',
				'field_event_starred' => '',
				'field_clone_source' => '',
				'field_event_edited_by' => ''
		);
	}

	/**
	 * Change the format of input taxonomy data
	 *
	 * @see Migration::prepareRow()
	 * @param
	 *        	$row
	 * @return bool Always import data
	 */
	public function prepareRow($row) {
		_cast_taxonomy_to_array ( $row );
		_update_textarea_links ( $row );
		_import_taxonomy ( $row->field_event_scope, $row );
		_merge_fields ( $row->field_event_language, $row->field_event_other_languages );
		return TRUE;
	}

	/**
	 * Change multiple array fields and rise it one level
	 * Import related files
	 *
	 * @param unknown $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array ( $node );
		_epractice_file_import ( $row->field_event_documentation [0], $node->field_event_documentation, $node->uid );
		_epractice_og_content ( $node, $row->field_event_domain_topics );
		_format_date_field ( ( object ) array (
				'value' => $row->field_event_start [0]->value,
				'value2' => $row->field_event_end [0]->value,
				'timezone' => $row->field_event_start [0]->timezone,
				'timezone_db' => $row->field_event_end [0]->timezone_db
		), $node->field_event_dates, 'date' );
		_add_urls ( $row->field_event_website [0], $node->field_event_website );
		_import_event_taxonomy_free_or_feee ( $row->field_event_free [0], $node->taxonomy, 'Free event' );
		_import_event_taxonomy_free_or_feee ( $row->field_event_open [0], $node->taxonomy, 'Open event' );
		_cast_import_taxonomy_field($row->field_event_language, $node->field_languages_fields, TRUE);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
      if($sid = _get_workflow_state($row->_workflow, $node->type)){
        workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
      }
	  _highlight_content($node, $row->field_event_domain_topics);
	  _update_changed_date($node->nid,$row->changed);
    }

	/**
	 * Generate string from an array
	 *
	 * @param unknown $value
	 * @return string
	 */
	function getStringFromArray($value) {
		if (is_array ( $value )) {
			return $value [0]->value;
		}

		return $value;
	}

	/**
	 *
	 * @param unknown $value
	 * @return string
	 */
	function getLowerCase($value) {
		return strtolower ( $value );
	}
}