<?php

/**
 * Handler to migrate communities from Epractice website
 * @author antartem
 *
 */
class JoinupCaseMigration extends Migration {
  /**
   * Setup the field mappings between the source and destination
   */
  public function __construct() {
    parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));
    $json_file = MIGRATION_FILES_JSON . '/joinup_cases.json';
    $this->map = new MigrateSQLMap($this->machineName,
      array(
        'nid' => array(
          'type' => 'int',
          'unsigned' => TRUE,
          'not null' => TRUE,
        ),
      ),
      MigrateDestinationNode::getKeySchema()
    );

    $this->source = new MigrateSourceJSON($json_file, 'nid', $this->fields());
    $this->destination = new MigrateDestinationNode('case_epractice');
    $this->addSimpleMappings(array('title', 'body', 'created', 'format', 'status', 'language'));
  }

  /**
   * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
   * @return array
   */
  function fields() {
    return array(
      'nid' => 'Node id',
      'uid' => 'User id',
      'language' => '',
	  'format' => '',
      'created' => '',
      'changed' => '',
      'status' => '',
      'title' => '',
      'body' => '',
      'comment' => '',
      'promote' => '',
      'moderate' => '',
      'sticky' => '',
      'translate' => '',
      'field_document_study' => '',
      'taxonomy' => ''
    );
  }

  /**
   * Change the format of input taxonomy data
   * @see Migration::prepareRow()
   * @param $row
   * @return bool Always import data
   */
  public function prepareRow($row) {
    _cast_taxonomy_to_array($row);
    return TRUE;
  }

  /**
   * Change multiple array fields and rise it one level
   * Import related files
   * @param stdClass $node
   * @param stdClass $row
   */
  public function prepare($node, stdClass $row) {
    _cast_node_object_to_array($node);
    $node->type = "case_epractice";
    $node->taxonomy = $row->taxonomy;
    $node->uid = $row->uid;
    foreach($row->field_document_study as $field_document_study){
      $field_document_study = (array)$field_document_study;
      unset($field_document_study['data']);
      $node->field_case_documentation[] = $field_document_study;
    }
	_update_changed_date($node->nid,$row->changed);
  }


  /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
		workflow_execute_transition($node, $row->_workflow, 'Automatic Epractice Migration Process', TRUE);
		_update_changed_date($node->nid,$row->changed);
	}
}