<?php

/**
 * Handler to migrate news from Epractice website
 * @author antartem
 *
 */
class EpracticeNewsMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		//
		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));

		$json_file =  MIGRATION_FILES_JSON.'/news.json';
		///die($json_file);
		//no options, or you could specify some
		$http_options = array();
		//map for the migration
		$this->map = new MigrateSQLMap($this->machineName,
			array(
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				),
			),
			MigrateDestinationNode::getKeySchema()
		);

		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';

		/*
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, 'nid'),
				new EpracticeItemJSON($json_file, $http_options, 'nid'),
				$this->fields()
		);
		*/
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());

		$this->destination = new MigrateDestinationNode('news');

		$this->addFieldMapping('uid', 'uid')->defaultValue(0)
			->sourceMigration('EpracticeUser');
		$this->addSimpleMappings(array('title', 'created', 'format', 'status', 'language'));
		$this->addFieldMapping('teaser', 'field_news_summary');
		$this->addFieldMapping('body', 'field_news_description');
		$this->addFieldMapping('field_city', 'field_news_city')
		  ->callbacks(array($this, 'getStringFromArray'));
		$this->addFieldMapping('field_email', 'field_news_email')
		  ->callbacks(array($this, 'getEmail'));
		$this->addFieldMapping('taxonomy', 'taxonomy')
		->sourceMigration('EpracticeTaxonomy')
		->arguments(array('source_type' => 'tid'));
	}


	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
			//'nid' => 'Node id',
			'uid' => 'User id',
			'language' => '',
			'format' => '',
			'created' => '',
			'changed' => '',
			'status' => '',
			'title' => '',
			'body' => '',
			'teaser' => '',
			'og_groups' => '',
			'field_news_summary' => '',
			'field_news_description' => '',
			'field_news_domain' => '',
  		  'field_news_topic' => '',
  		  'field_news_country' => '',
  		  'field_news_city' => '',
  		  'field_news_source' => '',
  		  'field_news_email' => '',
  		  'field_news_documentation' => '',
  		  'field_news_uploaded' => '',
  		  'field_news_tags' => '',
  		  'field_news_sector' => '',
  		  'field_news_edited_by' => '',
  		  'field_news_node_revision_date' => '',
  		  'field_news_node_revision_comm' => '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_og_groups_to_array($row);
		_cast_taxonomy_to_array($row);
		_import_sector_taxonomy($row);
 		_update_textarea_links($row);

		return TRUE;
	}

	/**
	 * Change multiple array fields and rise it one level
	 * Import related files
	 * @param unknown $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		_epractice_file_import($row->field_news_documentation, $node->field_documentation, $node->uid);
		_epractice_og_content($node, $row->field_news_domain);
		_add_urls($row->field_news_source[0], $node->field_source_url);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
      if($sid = _get_workflow_state($row->_workflow, $node->type)){
        workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
      }
	  _highlight_content($node, $row->field_news_domain);
	  foreach($row->og_groups as $gid){
		$results = db_query("SELECT destid1 FROM migrate_map_epracticecommunity WHERE sourceid1=%d",$gid);
		if($destid = db_fetch_array($results)){
	      _highlight_content($node, $destid['destid1']);
		}
	  }
	  _update_changed_date($node->nid,$row->changed);
    }

	/**
	 * Generate string from an array
	 * @param unknown $value
	 * @return string
	 */
	function getStringFromArray($value){
	  if(is_array($value)){
	    return $value[0]->value;
	  }

	  return $value;
	}

	/**
	 * Generate the email
	 * @param unknown $value
	 * @return string
	 */
	function getEmail($value){
	 	if(is_array($value)){
	    return $value[0]->email;
	  }

	  return $value;
	}

}