<?php

/**
 * Handler to migrate user from Epractice website
 * @author antartem
 *
 */
class EpracticeProfileMigration extends Migration {
	public $stored_rules;

	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		//
		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));

		$json_file =  MIGRATION_FILES_JSON.'/user.json';
		$this->map = new MigrateSQLMap($this->machineName,
			array(
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				),
			),
			MigrateDestinationNode::getKeySchema()
		);

		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());
		
		$this->destination = new MigrateDestinationNode('profile');

		$this->addFieldMapping('uid', 'uid')
			->sourceMigration('EpracticeUser');
		$this->addFieldMapping('format', 'format');
		$this->addFieldMapping('created', 'created');
		$this->addFieldMapping('field_firstname', 'profile_name');
		$this->addFieldMapping('field_lastname', 'profile_surname');
		$this->addFieldMapping('field_company_name', 'profile_organisation');
		$this->addFieldMapping('field_company_scope', 'profile_organisation_scope');
		$this->addFieldMapping('field_company_type', 'profile_organisation_type');
		$this->addFieldMapping('field_zipcode', 'profile_org_contact_postcode');
		$this->addFieldMapping('field_city', 'profile_org_contact_city');
		$this->addFieldMapping('field_street', 'profile_org_contact_street');
		//$this->addFieldMapping('field_company_country', 'profile_org_contact_country');
		$this->addFieldMapping('field_company_phone', 'profile_org_contact_telephone');
		$this->addFieldMapping('field_professional_profile', 'field_short_bio');

		$this->addFieldMapping('field_professional_p_visibility', 'profile_privacy_professional_profile');
		$this->addFieldMapping('field_email_visibility', 'profile_iagree');
		$this->addFieldMapping('field_profile_visibility', 'profile_iagree');
		$this->addFieldMapping('field_keywords_visibility', 'profile_iagree');
		$this->addFieldMapping('field_country_visibility', 'profile_iagree');
		$this->addFieldMapping('field_company_visibility', 'profile_iagree');
		$this->addFieldMapping('field_facebook_visibility', 'profile_iagree');
		$this->addFieldMapping('field_twitter_visibility', 'profile_iagree');
		$this->addFieldMapping('field_linkedin_visibility', 'profile_iagree');

 		$this->addFieldMapping('field_public_speaker', 'field_public_speaker');

		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
	}


	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'format' => '',
				'created' => '',
				'changed' => '',
				'mail' => '',
				'profile_name' => '',
				'profile_surname' => '',
				'profile_country' => '',
				'profile_organisation' => '',
				'profile_organisation_scope' => '',
				'profile_organisation_type' => '',
				'profile_org_contact_postcode' => '',
				'profile_org_contact_city' => '',
				'profile_org_contact_street' => '',
				'profile_org_contact_country' => '',
				'profile_org_contact_telephone' => '',
				'field_short_bio' => '',
				'field_upload_cv' => '',
				'profile_privacy_professional_profile' => '',
				'profile_iagree' => '',
				'field_linkedin_account' => '',
				'field_expertise_keywords1' => '',
				'field_expertise_keywords2' => '',
				'field_expertise_keywords3' => '',
				'field_languages' => '',
				'field_other_languages' => '',
				'field_domain_work',
                'field_your_sector',
                'field_topics_of_interest',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		$mail = $row->mail;
		$mail = ltrim($mail,"_");
		$row->mail = $mail;
		if ($user = user_load(array('mail'=> $row->mail))) {
			$query_result = db_query("SELECT nid FROM node WHERE uid = %d AND type = 'profile'",$user->uid);
			if($nid = db_fetch_array($query_result)){
				$this->rollbackAction = MigrateMap::ROLLBACK_PRESERVE;
				$this->map->saveIDMapping($row, array($nid['nid']), MigrateMap::STATUS_IGNORED, MigrateMap::ROLLBACK_PRESERVE);
				_update_profile($nid['nid'],$row);
				return FALSE;
			}
		}
		$this->rollbackAction = MigrateMap::ROLLBACK_DELETE;
		
		_cast_taxonomy_to_array($row);
 		_update_textarea_links($row);
		return TRUE;
	}

  /**
   *
   */
 /* public function preRollback(){
      parent::preRollback();
      $map_table = $this->map->getMapTable();
      // Get IDs of all categories that were imported
      $query_result = db_query('select destid1 from ' . $map_table .' where destid1 is not null and rollback_action = '.MigrateMap::ROLLBACK_DELETE);
      while ($row = db_fetch_array($query_result)) {
        _restore_file('field_photo', 'profile', $row['destid1']);
        _restore_file('field_profile_cv', 'profile', $row['destid1']);
      }
  }*/

	/**
	 * Change multiple array fields and rise it one level
	 * @param stdClass $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		$node->title = sprintf ('%s %s', $node->field_firstname[0]['value'], $node->field_lastname[0]['value']);
		_epractice_file_import($row->field_upload_cv, $node->field_profile_cv, $node->uid);
		$picture = $row->picture;
		if(!$picture || $picture == "sites/all/modules/DRUPAU_people/img/predefined.jpg"){
		  //Load the default
          $default_picture = isa_toolbox_default_imagefield('field_photo', 'profile');
		  $node->field_photo[] = $default_picture;
		}else{
          _epractice_file_import($row->picture, $node->field_photo, $node->uid);
		}
		_cast_import_taxonomy_field($row->field_languages, $node->field_languages_fields, TRUE);
		_import_other_language($row->field_other_languages,$node->field_languages_fields);
        _cast_import_taxonomy_field($row->field_your_sector, $node->field_profile_sector);
        $topics = array();
        _cast_import_taxonomy_field($row->field_topics_of_interest, $topics);
        foreach($topics as $topic) { $node->taxonomy[] = $topic['value']; }
		$company_countries = array();
		$company_countries = _convert_string_to_terms($row->profile_org_contact_country,",",26);
		foreach($company_countries as $tid){
		  $node->field_country[] = array("value"=>$tid);
		} 
		$nationalities = array();
		$nationalities = _convert_string_to_terms($row->profile_country,",",26);
		foreach($nationalities as $tid){
		  $node->taxonomy[] = $tid;
		} 
		$expertise_keywords = array();
		$expertise_keywords = array_merge($expertise_keywords,_convert_string_to_terms($row->field_expertise_keywords1[0]->value,",",28,true));
		$expertise_keywords = array_merge($expertise_keywords,_convert_string_to_terms($row->field_expertise_keywords2[0]->value,",",28,true));
		$expertise_keywords = array_merge($expertise_keywords,_convert_string_to_terms($row->field_expertise_keywords3[0]->value,",",28,true));
		foreach($expertise_keywords as $keyword) { $node->taxonomy[] = $keyword; }
		_add_linkedin_account($node->field_linkedin,$row->field_linkedin_account[0]->value);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
		if(isset($node->field_profile_sector)&&is_array($node->field_profile_sector)&&isset($node->taxonomy)&&is_array($node->taxonomy)){
		  foreach($node->field_profile_sector as $sector){
		    foreach($node->taxonomy as $key=>$term){
		      if($sector['value']==$term){
			    unset($node->taxonomy[$key]);
			  }
		    }
		  }
		}
    }
	
	/**
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
	  _update_changed_date($node->nid,$row->changed);
    }
}
