<?php

/**
 * Handler to migrate user from Epractice website
 * @author antartem
 *
 */
class EpracticeResourceMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		parent::__construct ( MigrateGroup::getInstance ( 'isa_epractice_migration' ) );

		$json_file = MIGRATION_FILES_JSON.'/resource.json';

		// Human-friendly description of your migration process. Be as detailed as you
		// like.
		$this->description = t ( 'Migrate Resource nodes to Joinup' );

		$this->map = new MigrateSQLMap ( $this->machineName, array (
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				)
		), MigrateDestinationTerm::getKeySchema () );


		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());
		
		$this->destination = new MigrateDestinationNode('document');
		$this->addFieldMapping('uid', 'uid')
			->sourceMigration('EpracticeUser');
		$this->addFieldMapping('language', 'language');
		$this->addFieldMapping('created', 'created');
		$this->addFieldMapping('format', 'format');
		$this->addFieldMapping('status', 'status');
		$this->addFieldMapping('title', 'title');
		$this->addFieldMapping('body', 'body');
		//$this->addFieldMapping('og_groups', 'og_groups')
			//->sourceMigration('EpracticeCommunity'));
		$this->addFieldMapping('field_additional_doc_desc', 'field_resources_alternativetxt');
		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'format' => '',
				'created' => '',
				'changed' => '',
				'status' => '',
				'title' => '',
				'body' => '',
				'teaser' => '',
				'field_resource_city'=> '',
				'field_resource_domain'=> '',
				'field_resource_file'=> '',
				'field_resource_license'=> '',
				'field_resource_long'=> '',
				'field_resource_organisation'=> '',
				'field_resource_spread'=> '',
				'field_resource_sector'=> '',
				'field_resource_type'=> '',
				'field_resource_url'=> '',
				'field_resource_visibility'=> '',
				'field_resource_tags'=> '',
				'field_resource_country'=> '',
				'field_resource_language'=> '',
				'field_resource_topic'=> '',
				'og_groups'=> '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_og_groups_to_array($row);
		_cast_taxonomy_to_array($row);
 		_update_textarea_links($row);
		return TRUE;
	}

	/**
	 * Modify row and node objects
	 * @param unknown $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		_epractice_file_import($row->field_resources_upload, $node->field_additional_doc_file, $node->uid);
		_epractice_og_content($node, NULL);
		_add_urls($row->field_external_url[0], $node->field_original_url);
		_cast_import_taxonomy_field($row->field_resource_language, $node->field_languages_fields, TRUE);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
      if($sid = _get_workflow_state($row->_workflow, $node->type)){
        workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
      }
	  _highlight_content($node, $row->field_resource_domain);
	  _update_changed_date($node->nid,$row->changed);
    }

}
