<?php

/**
 * Handler class to migrate taxonomy terms
 * @author antartem
 *
 */
class EpracticeTaxonomyMigration extends Migration {
	/**
	 * Initialise the migrate engine
	 */
	public function __construct() {
		parent::__construct ( MigrateGroup::getInstance ( 'isa_epractice_migration' ) );

		$json_file =  MIGRATION_FILES_JSON.'/term.json';

		// Human-friendly description of your migration process. Be as detailed as you
		// like.
		$this->description = t ( 'Migrate vocabularies from the source json to taxonomy vocabularies' );

		// no options, or you could specify some
		$http_options = array ();

		$this->map = new MigrateSQLMap ( $this->machineName, array (
				'id' => array (
						'type' => 'varchar',
						'length' => 255
				)
		), MigrateDestinationTerm::getKeySchema () );


		/*
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, 'id'),
				new EpracticeItemJSON($json_file, $http_options, 'id'),
				$this->fields()
		);
		*/
		
		$this->source = new MigrateSourceJSON($json_file , 'id', $this->fields());
		
		$this->destination = new MigrateDestinationTerm('tags');
		$this->addFieldMapping('name', 'name');
		$this->addFieldMapping('parent', 'parent_id')->sourceMigration($this->machineName);
		$this->addFieldMapping('vocabulary_machine_name', 'vocab');
	}

	/**
	 * Map epractice and joinup vocabulary names
	 * @see Migration::prepareRow()
	 * @param $row stdClass
	 * @return bool FALSE if not match the vocab name, TRUE if match
	 */
	public function prepareRow(stdClass $row){

		/** var array Map of same Terms with different strings **/
		$map_terms = array(
			'2665' => '1548',
			'2889' => '1544',
			'220' => '1345',
			'220' => '1345',
			'210' => '1338',
			'367' => '1385',
			'370' => '1388',
			'200' => '1325',
			'196' => '1321',
			'2' => '709',
			'15' => '15866',
			'40' => '718',
			'41' => '719',
			'42' => '720',
			'32' => '15857',
		);
	
		/** var array Map of epractice and Joinup **/
		$map_vocabularies = array(
			'Factsheets Topic' => 'Factsheet topic',
			'Implementation' => 'Implementation cost',
			'Languages' => 'My languages',
			'Funding source' => 'Funding source',
			'Overall Implementation approach' => 'Overall implementation approach',
			'Scope' => 'Scope',
			'Yearly cost' => 'Yearly cost',
			'Technology choice' => 'Technology choice',
			'Type of initiative' => 'Type of initiative',
			'Status' => 'Case status',
			'Countries' => 'Geographic coverage',
			'organisation_type' => 'Organisation type',
			'documentation nature' => 'Nature of documentation',
			'Topics of interest' => 'Themes',
			'ePractice domain' => 'Themes',
			'Tags' => 'Keywords',
			'Sector' => 'Themes',
			'Economic effects' => 'Return on investment',
			'Target Users or Group' => 'Target Users or Group',
			'Type of initiative - B' => 'Type of service'
		);

		if (isset($map_terms[$row->id])) {
			$this->rollbackAction = MigrateMap::ROLLBACK_PRESERVE;
			$this->map->saveIDMapping($row, (array)$map_terms[$row->id], MigrateMap::STATUS_IGNORED, MigrateMap::ROLLBACK_PRESERVE);

			return FALSE;
		}
		
		if(isset($map_vocabularies[$row->vocab])) {
			// Change the name of vocabulary from epractice to joinup names
			$row->vocab = $map_vocabularies[$row->vocab];
			if (!_get_vocabulary_by_name($row->vocab)){
				$vocabulary = array(
						'name' => t($row->vocab), //Human readable name of the vocabulary
						'multiple' => TRUE, //set 1 to allow multiple selection
						'required' => FALSE, //set 1 to make the terms mandatory to be selected
						'hierarchy' => TRUE, //set 1 to allow and create hierarchy of the terms within the vocabulary
						'relations' => TRUE, //set 1 to set and allow relation amongst multiple terms
						'module' => 'isa_epractice_migration', //provide the module name in which the vocabulary is defined and which is calling this function
						'nodes' => array(), //TODO: set the node to which this vocabulary will be attached to
						'weight' => -10, //set the weight to display the vocabulary in the list
				);
				taxonomy_save_vocabulary($vocabulary); //saving the $vocabulary array will create a vocabulary.
			}
			$terms = taxonomy_get_term_by_name($row->name);
			$vocabs = taxonomy_get_vocabularies();
			foreach($terms as $term){
			  $vocab = $vocabs[$term->vid];
			  //Term exists in vocabulary, don't import it
			  if($vocab->name == $row->vocab){
			    $this->rollbackAction = MigrateMap::ROLLBACK_PRESERVE;
				$this->map->saveIDMapping($row, (array)$term->tid, MigrateMap::STATUS_IGNORED, MigrateMap::ROLLBACK_PRESERVE);
				return FALSE;
			  }
			}
			$this->rollbackAction = MigrateMap::ROLLBACK_DELETE;
			return TRUE;
		}

		// Skip the current row if the current vocabulary is not in the list
		return FALSE;
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'id' => '',
				'name' => '',
				'parent_id' => '',
				'vocab' => ''
		);
	}
}


