<?php

/**
 * Handler to migrate user from Epractice website
 * @author antartem
 *
 */
class EpracticeTvMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		parent::__construct ( MigrateGroup::getInstance ( 'isa_epractice_migration' ) );

		$json_file = MIGRATION_FILES_JSON.'/tv.json';

		// Human-friendly description of your migration process. Be as detailed as you
		// like.
		$this->description = t ( 'Migrate Resource nodes to Joinup' );

		// no options, or you could specify some
		$http_options = array ();

		$this->map = new MigrateSQLMap ( $this->machineName, array (
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				)
		), MigrateDestinationTerm::getKeySchema () );


		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';

		/*
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, 'nid'),
				new EpracticeItemJSON($json_file, $http_options, 'nid'),
				$this->fields()
		);
		*/
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());
		
		$this->destination = new MigrateDestinationNode('video');
		$this->addFieldMapping('uid', 'uid')
			->sourceMigration('EpracticeUser')
			->defaultValue(0);
		$this->addFieldMapping('language', 'language');
		$this->addFieldMapping('created', 'created');
		$this->addFieldMapping('format', 'format');
		$this->addFieldMapping('status', 'status');
		$this->addFieldMapping('title', 'title');
		$this->addFieldMapping('body', 'body');
		$this->addFieldMapping('field_tv_country', 'field_tv_country')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));	
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'format' => '',
				'created' => '',
				'changed' => '',
				'status' => '',
				'title' => '',
				'body' => '',
				'teaser' => '',
				'field_tv_country'=> '',
				'field_tv_date'=> '',
				'field_tv_domain'=> '',
				'field_tv_tags'=> '',
				'field_tv_topic'=> '',
				'field_tv_youtube_video'=> '',
				'og_groups'=> '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_og_groups_to_array($row);
		_cast_taxonomy_to_array($row);
		_cast_field_to_array(&$row->field_tv_country);
 		_update_textarea_links($row);
		return TRUE;
	}

	/**
	 * Modify row and node objects
	 * @param unknown $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		_cast_node_object_to_array($node);
		_epractice_og_content($node, NULL);
		_epractice_embed_video($node,$row->field_tv_youtube_video[0]);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
	}
	
	 /**
     * Highlight node in community based on domain
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
	  _highlight_content($node, $row->field_tv_domain);
	  _update_changed_date($node->nid,$row->changed);
    }
}
