<?php

use Common\Object;
/**
 * Handler to migrate user from Epractice website
 * @author antartem
 *
 */
class EpracticeUserMigration extends Migration {

	/** var $ROLES array Array with mapping from ePractice to Joinup roles **/
	protected $ROLES = array(
		2 => 2, // authenticated user -> authenticated user
		10 => 9, // developer -> developer
		16 => 3, // EC -> moderator
		17 => 3, // ED -> moderator
		8 => 3, // editor -> moderator
		7 => 3, // editor European Dinamics -> moderator
	);

	protected $ROLE_NAMES = array(
			2 => 'authenticated user', // authenticated user -> authenticated user
			9 => 'developer', // developer -> developer
			3 => 'moderator', // EC -> moderator
	);

	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));


		$json_file =  MIGRATION_FILES_JSON.'/user.json';
		///die($json_file);
		//no options, or you could specify some
		$http_options = array();
		//map for the migration
		$this->map = new MigrateSQLMap($this->machineName,
			array(
				'uid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				),
			),
			MigrateDestinationUser::getKeySchema()
		);

		/*
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, 'uid'),
				new EpracticeItemJSON($json_file, $http_options, 'uid'),
				$this->fields()
		);
		*/
		
		$this->source = new MigrateSourceJSON($json_file , 'uid', $this->fields());

		$this->destination = new MigrateDestinationUser();

		$this->addFieldMapping('name', 'name')->dedupe('users', 'name');
		$this->addSimpleMappings( array('pass', 'mail', 'mode', 'sort', 'treshold', 'theme', 'signature',
				'created', 'access', 'login', 'status', 'timezone',  'language', 'init', 'data', 'timezone_name')
		);


		$this->addFieldMapping('roles')->defaultValue(array(DRUPAL_AUTHENTICATED_RID));
		$this->addFieldMapping('role_names', 'role_name')->defaultValue('authenticated user');

		// Other handlers
		$this->addFieldMapping('path')
		->defaultValue(NULL);
		$this->addFieldMapping('pathauto')
		->defaultValue(''); // Perform aliasing (set to 0 to prevent alias generation during migration)
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'uid' => 'User id',
				'name' => '',
				'pass' => '',
				'mail' => '',
				'mode' => '',
				'sort' => '',
				'treshold' => '',
				'theme' => '',
				'signature' => '',
				'signature_format' => '',
				'created' => '',
				'access' => '',
				'login' => '',
				'status' => '',
				'timezone' => '',
				'language' => '',
				'picture' => '',
				'init' => '',
				'data' => '',
				'timezone_name' => '',
				'field_domain_work'=>'',
				
		);
	}

    /**
	 * Map epractice roles to joinup roles
	 * Updates the created date to the current time
	 * https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1233
     */
    public function prepare($account, $row) {
		$roles = (array) $row->roles;
		$account->roles = array();
		$account->role_names = array();
		$setted_roles = array();

		foreach ($roles as $role_id => $role_name){
			if(isset($this->ROLES[$role_id]) && !isset($setted_roles[$this->ROLES[$role_id]])) {
				$setted_roles[$this->ROLES[$role_id]] = TRUE;

				$account->roles[$this->ROLES[$role_id]] = $this->ROLES[$role_id];
				$account->role_names[$this->ROLES[$role_id]] = $this->ROLE_NAMES[$this->ROLES[$role_id]];
			}
		}

		$account->created = time();
	}

    /**
     * Import kudos for the user
     * @param unknown $entity
     * @param unknown $row
     */
	function complete($entity,$row){
	  _subscribe_user($entity,$row->field_domain_work);
	  _subscribe_user($entity,variable_get('community_epractice_epractice',0));
	  $params = array (
        'uid' => $entity->uid,
        'points' => $row->kudos,
      );
      userpoints_userpointsapi($params);
	  
	  
	}

	/**
	 * Save id mapping in case if user email already exist on Joinup
	 * @see Migration::prepareRow()
	 * @param stdClass $row
	 */
	public function prepareRow(stdClass $row) {
		$mail = $row->mail;
		$mail = ltrim($mail,"_");
		$row->mail = $mail;
		if ($user = user_load(array('mail'=> $row->mail))) {
			$this->rollbackAction = MigrateMap::ROLLBACK_PRESERVE;
			$this->map->saveIDMapping($row, array($user->uid), MigrateMap::STATUS_IGNORED, MigrateMap::ROLLBACK_PRESERVE);
			_subscribe_user($user,$row->field_domain_work);
			_subscribe_user($user,variable_get('community_epractice_epractice',0));
			$params = array (
			  'uid' => $user->uid,
			  'points' => $row->kudos,
			);
			userpoints_userpointsapi($params);
			return FALSE;
		}
		$this->rollbackAction = MigrateMap::ROLLBACK_DELETE;
		
		return TRUE;
	}


	/**
	 * Handle duplicated usernames
	 * Add _eP suffix to the name of the user
	 * @see Migration::handleDedupe()
	 */
	protected function handleDedupe(array $dedupe, $original) {
		$original_changed = parent::handleDedupe($dedupe, $original);

		if($original != $original_changed) {
			$original .= '_eP';
		}

		return $original;
	}

}
