<?php

/**
 * Handler to migrate user associations to community content type organic groups from Epractice website
 * @author antartem
 *
 */
class EpracticeUserCommunityRelationMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {
		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));
		$json_file =  MIGRATION_FILES_JSON.'/og_membership.json';
		$http_options = array();
		$source_key = array(
			'gid' => array(
				'type' => 'int',
				'not null' => TRUE,
				'description' => 'NID of group',
			),
			'uid' => array(
				'type' => 'int',
				'not null' => TRUE,
				'description' => 'UID of member',
			),
		);
		$this->map = new MigrateSQLMap($this->machineName, $source_key,
			MigrateDestinationOGMembership::getKeySchema()
		);


		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';
		
		$this->source = new MigrateSourceList(
				new EpracticeListJSON($json_file, $http_options, array('gid', 'uid')),
				new EpracticeItemJSON($json_file, $http_options, array('gid', 'uid')),
				$this->fields()
		);

		$this->destination = new MigrateDestinationOGMembership();
		$this->addSimpleMappings(array('is_admin', 'created', 'is_active'));
		$this->addFieldMapping('gid','gid')->sourceMigration('EpracticeCommunity');
		$this->addFieldMapping('uid', 'uid')->sourceMigration('EpracticeUser');
	}

	/**
	 * (non-PHPdoc)
	 * @see Migration::prepareRow()
	 */
	public function prepareRow($row) {
		$row->state = !empty($row->state) ? OG_STATE_ACTIVE : OG_STATE_PENDING;
		parent::prepareRow($row);
	}

	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'gid' => 'User id',
				'uid' => '',
				'created' => '',
				'is_active' => '',
				//'rid' => '',
		);
	}

}
