<?php

/**
 * Handler to migrate workshops from Epractice website
 * @author antartem
 *
 */
class EpracticeWorkshopMigration extends Migration {
	/**
	 * Setup the field mappings between the source and destination
	 */
	public function __construct() {

		parent::__construct(MigrateGroup::getInstance('isa_epractice_migration'));

		$json_file =  MIGRATION_FILES_JSON.'/workshops.json';
		$this->map = new MigrateSQLMap($this->machineName,
			array(
				'nid' => array(
					'type' => 'int',
					'unsigned' => TRUE,
					'not null' => TRUE,
				),
			),
			MigrateDestinationNode::getKeySchema()
		);

		$this->dependencies[] = 'EpracticeTaxonomy';
		$this->dependencies[] = 'EpracticeUser';
		$this->dependencies[] = 'EpracticeCommunity';
		
		$this->source = new MigrateSourceJSON($json_file , 'nid', $this->fields());
		
		$this->destination = new MigrateDestinationNode('workshop');
		$this->addFieldMapping('uid', 'uid')->defaultValue(0)
			->sourceMigration('EpracticeUser');
		$this->addSimpleMappings(array('title', 'body', 'teaser', 'status','language', 'format', 'created'));
		$this->addFieldMapping('field_event_city', 'field_ew_city');
		$this->addFieldMapping('field_event_venue', 'field_ew_venue');
		$this->addFieldMapping('field_event_address_location', 'field_ew_address');
		$this->addFieldMapping('field_event_gmap_location', 'field_ew_gmap_location');
		$this->addFieldMapping('field_event_organiser', 'field_ew_organiser');
		$this->addFieldMapping('field_event_website', 'field_ew_website');
		$this->addFieldMapping('field_event_contact_email', 'field_ew_contact_email');
		$this->addFieldMapping('field_event_agenda', 'field_ew_agenda');
		$this->addFieldMapping('field_event_exp_participants', 'field_ew_expected_participants');
		$this->addFieldMapping('field_event_fees_description', 'field_ew_fees_description');


		$this->addFieldMapping('taxonomy', 'taxonomy')
			->sourceMigration('EpracticeTaxonomy')
			->arguments(array('source_type' => 'tid'));
	}


	/**
	 * Return the fields (this is cleaner than passing in the array in the MigrateSourceList class above)
	 * @return array
	 */
	function fields() {
		return array(
				'nid' => 'Node id',
				'uid' => 'User id',
				'language' => '',
				'status' => '',
				'title' => '',
				'format' => '',
				'created' => '',
				'changed' => '',
				'body' => '',
				'teaser' => '',
				'field_ew_logo' => '',
				'field_ew_languages' => '',
				'field_ew_start_date' => '',
				'field_ew_end_date' => '',
				'field_ew_city' => '',
				'field_ew_venue' => '',
				'field_ew_address' => '',
				'field_ew_gmap_location' => '',
				'field_ew_organiser' => '',
				'field_ew_website' => '',
				'field_ew_contact_email' => '',
				'field_ew_agenda' => '',
				'field_ew_expected_participants' => '',
				'field_ew_fees_description' => '',
				'field_ew_free_event' => '',
				'field_ew_open_event' => '',
				'field_ew_domain' => '',
				'og_groups'=> '',
		);
	}

	/**
	 * Change the format of input taxonomy data
	 * @see Migration::prepareRow()
	 * @param $row
	 * @return bool Always import data
	 */
	public function prepareRow($row){
		_cast_og_groups_to_array($row);
		_cast_taxonomy_to_array($row);
		_update_textarea_links($row);
		return TRUE;
	}

	/**
	 * Change multiple array fields and rise it one level
	 * Import related files
	 * @param stdClass $node
	 * @param stdClass $row
	 */
	public function prepare($node, stdClass $row) {
		$node->type = 'event';
		_cast_node_object_to_array($node);
		_epractice_file_import($row->field_ew_logo, $node->field_event_logo, $node->uid);
		_epractice_og_content($node, $row->field_ew_domain);
		_format_date_field((object) array(
				'value' => $row->field_ew_start_date[0]->value,
				'value2' => $row->field_ew_end_date[0]->value,
				'timezone' => $row->field_ew_start_date[0]->timezone,
				'timezone_db' => $row->field_ew_start_date[0]->timezone_db
			),
			$node->field_event_dates,
			'date'
		);
		_import_event_taxonomy_free_or_feee($row->field_ew_free_event[0], $node->taxonomy, 'Free event');
		_import_event_taxonomy_free_or_feee($row->field_ew_open_event[0], $node->taxonomy, 'Open event');
		_cast_import_taxonomy_field($row->field_ew_languages, $node->field_languages_fields, TRUE);
		_add_urls($row->field_ew_website[0], $node->field_event_website);
		if($node->taxonomy){
		  $node->taxonomy = array_unique($node->taxonomy);
		}
	}

    /**
     * Import current workflow state
     * @param $node
     * @param stdClass $row
     */
    public function complete($node, stdClass $row){
      if($sid = _get_workflow_state($row->_workflow, $node->type)){
        workflow_execute_transition($node, $sid, 'Automatic Epractice Migration Process', TRUE);
      }
	  _highlight_content($node, $row->field_ew_domain);
	  foreach($row->og_groups as $gid){
		$results = db_query("SELECT destid1 FROM migrate_map_epracticecommunity WHERE sourceid1=%d",$gid);
		if($destid = db_fetch_array($results)){
	      _highlight_content($node, $destid['destid1']);
		}
	  }
	  _update_changed_date($node->nid,$row->changed);
    }


}