<?php
 /**
 * @defgroup isa_nifo
 * @brief NIFO analysis helpers
 * @file
 * Modules main file
 */
/**
* returns an array with countries that have at least one active analytical model
* It should show only countries with active analytical models
* @return: countries array
*/
function isa_nifo_analytical_getCountriesWithAM() {
  $rs = db_query("SELECT td.tid, td.name from {term_data} td
    , {term_node} tn, {workflow_node} wfn
    , {content_type_analytical_model} ctam
    WHERE
    td.tid = tn.tid and
    wfn.sid = %d and
    tn.nid = wfn.nid and
    ctam.nid = tn.nid and
    td.vid = %d
    GROUP BY tn.tid
    ORDER BY td.name, wfn.stamp DESC", variable_get('isa_nifo_workflow_published', ''), variable_get('isa_nifo_country_vid', ''));
  while ($element_row = db_fetch_object($rs)) {
    $countries[$element_row->tid]   = $element_row->name;
  }

  if (count($countries)>0) {
    return $countries;
  }
  else {
    return FALSE;
  }
}

/**
* Returns an array with de nifo categories
* @return: categories array
*/
function isa_nifo_analytical_getCategories() {
  $cats  = db_query("SELECT * FROM {nifo_categories} ORDER BY {nifo_categories}.order asc");
  while ( $row = db_fetch_object($cats) ) {
        //$categories[$row->idnifo_categories] = $row->nifo_categories_name;
        // To name the categories is better concatenated the order instead the identificator
    $categories[$row->idnifo_categories]['name'] = $row->nifo_categories_name;
    $categories[$row->idnifo_categories]['order'] = $row->order;
    $els = db_query("SELECT * FROM {nifo_eif_element} WHERE nifo_categories_idnifo_categories=%d ORDER BY {nifo_eif_element}.order asc", $row->idnifo_categories);
    while ($element_row = db_fetch_object($els)) {
      $elements[$element_row->idnifo_eif_element]
      = array(
        'nifo_categories_idnifo_categories' => $element_row->nifo_categories_idnifo_categories,
        'nifo_eif_element'                  => $element_row->nifo_eif_element,
        'nifo_eif_measurement'              => $element_row->nifo_eif_measurement,
        'nifo_eif_reference'                => $element_row->nifo_eif_reference,
        );
      $relations[$row->idnifo_categories][] = $element_row->idnifo_eif_element;
    }
  }
  if (count($categories)>0 && count($elements)) {
    return array(
      'categories'    => $categories,
      'elements'      => $elements,
      'relation'      => $relations,
      );
  }
  else {
    return FALSE;
  }
}

/**
 * Returns the last analytical model.
 * It comes as a variable belonging to the country taxonomy.
 *
 * The required attributes are:
 * - "countries" country taxonomy
 * @return models_by_country array
 * @ingroup isa_nifo
 *
 */
function isa_nifo_anaylytical_last_model($countries) {
  if (!is_array($countries)) {
    $countries[] = $countries;
  }
    // I wasn't taking the last analytical model of each country
    //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1051
    //Change the sql
   $models = db_query("SELECT *
       FROM (SELECT
          term_data.name,
          term_node.tid,
          term_node.nid,
          workflow_node.stamp,
          content_type_analytical_model.field_analytical_date_value
        FROM
          term_node,
          term_data,
          workflow_node,
          content_type_analytical_model
        WHERE
          term_data.tid = term_node.tid
            AND content_type_analytical_model.nid = term_node.nid
            AND term_node.tid in (%s)
            AND workflow_node.sid = %d
            AND term_node.nid = workflow_node.nid
        GROUP BY term_node.tid, content_type_analytical_model.field_analytical_date_value
        ORDER BY content_type_analytical_model.field_analytical_date_value DESC , workflow_node.stamp DESC
      ) result
      GROUP BY result.tid", implode(',', $countries), variable_get('isa_nifo_workflow_published'));
  while ($element_row = db_fetch_object($models)) {
    $tid = $element_row->tid;
    $nid = $element_row->nid;
    $models_by_country[$tid]['country'] = taxonomy_get_term($tid);
    $models_by_country[$tid]['node'] = node_load($nid);
  }
  return $models_by_country;
}


/**
* process de array data to return an array with average, etc
*/
function isa_nifo_analytical_process_data($las_analytical_model, $categories_nifo, $countries) {
  $data2['zone']['category']['element']['country'] = '';
  if (count($las_analytical_model) && count($categories_nifo)) {
    $categories = $categories_nifo['categories'];
    $elements   = $categories_nifo['elements'];
    foreach ($las_analytical_model as $analytical_model) {
      $country = $analytical_model['country'];
      $id_country = $country->tid;
      $node = $analytical_model['node'];
      $nifo_values = $node->nifo_values;
      $sw_cat = 'actual';
      foreach ($nifo_values as $values) {
        $id_category = $elements[$values->nifo_eif_element_idnifo_eif_element]['nifo_categories_idnifo_categories'];
        $id_element  = $values->nifo_eif_element_idnifo_eif_element;
        //if ($values->nifo_score_alignement!='') {
          $data[$id_country][$id_category][$id_element]['alignement']
          = array(
            'nifo_score_alignement'     => $values->nifo_score_alignement,
            'nifo_evidence_alignement'  => $values->nifo_evidence_alignement,
            'nifo_evidence_alignement'  => $values->nifo_evidence_alignement,
            );
          $data[$id_country][$id_category]['alignement']['sum'] = $data[$id_country][$id_category]['alignement']['sum'] + $values->nifo_score_alignement;
          $data[$id_country][$id_category]['alignement']['count'] = ($values->nifo_score_alignement!='')
          ? $data[$id_country][$id_category]['alignement']['count']+1
          : $data[$id_country][$id_category]['alignement']['count'];
//          : $data[$id_country][$id_category]['alignementalignement']['count'];
          $data2['alignement'][$id_category][$id_element][$id_country]
          = array(
            'nifo_score_alignement'     => $values->nifo_score_alignement,
            'nifo_evidence_alignement'  => $values->nifo_evidence_alignement,
            );

          // ISAICP-1103
          // Consider (-) as 0 for NIFO calculations
          $data[$id_country][$id_category]['alignement']['count_av'] = $data[$id_country][$id_category]['alignement']['count_av']+1;

          $data[$id_country][$id_category][$id_element]['monitoring']
          = array(
            'nifo_score_monitoring'     => $values->nifo_score_monitoring,
            'nifo_evidence_monitoring'  => $values->nifo_evidence_monitoring,
            );

          $data[$id_country][$id_category]['monitoring']['sum'] = $data[$id_country][$id_category]['monitoring']['sum'] + $values->nifo_score_monitoring;
          $data[$id_country][$id_category]['monitoring']['count'] = ($values->nifo_score_monitoring!='')
          ? $data[$id_country][$id_category]['monitoring']['count']+1
          : $data[$id_country][$id_category]['monitoring']['count'];
          // ISAICP-1103
          // Consider (-) as 0 for NIFO calculations
          $data[$id_country][$id_category]['monitoring']['count_av'] = $data[$id_country][$id_category]['monitoring']['count_av']+1;
          $data2['monitoring'][$id_category][$id_element][$id_country]
          = array(
            'nifo_score_monitoring'     => $values->nifo_score_monitoring,
            'nifo_evidence_monitoring'  => $values->nifo_evidence_monitoring,
            );
        //}
      }
    }
  }

  if (count($countries) && count($data)) {
    while (list($id_country, $sw) = each($data)) {

      while (list($idcat, $values) = each($sw)) {
        // ISAICP-1103
        // Consider (-) as 0 for NIFO calculations

        if( $values['alignement']['count'] == 0 ){
          $average['alignement'][$countries[$id_country]][$idcat] = '-';
        } else {
          //ISAICP-1070
          //NIFO comparisson should be in percentages

          $average['alignement'][$countries[$id_country]][$idcat] = round($values['alignement']['sum']/$values['alignement']['count_av'], 2, PHP_ROUND_HALF_UP);
        }
        if($values['monitoring']['count'] == 0 ){
          $average['monitoring'][$countries[$id_country]][$idcat] = '-';

        } else {
          //ISAICP-1070
          //NIFO comparisson should be in percentages
          $average['monitoring'][$countries[$id_country]][$idcat] = round($values['monitoring']['sum']/$values['monitoring']['count_av'], 2, PHP_ROUND_HALF_UP);
        }
      }
    }
  }
  $average_labels['data']['alignement'] = $average['alignement'];
  $average_labels['data']['monitoring'] = $average['monitoring'];
  // ISAICP-1066
  // Improvements on NIFO requested by Digit and Deloitte
  $average_labels['labels']['monitoring'] = t('NIF Implementation and Monitoring');
  $average_labels['labels']['alignement'] = t('EIF Alignment');
  return (array(
    'AVERAGE'       => $average_labels,
    'DATA'          => $data,
    'DATA_SUMMARY'  => $data2,
    ));
}

function isa_nifo_analytical_tableData($average, $categories, $exportpdf = false) {
  while (list($zone, $data) = each($average)) {
    $table[$zone] = isa_nifo_analytical_table($data, $categories, $zone, $exportpdf);
  }
  return $table;
}
function isa_nifo_analytical_table($data, $categories, $zone, $exportpdf = false) {
  $colours = _isa_export_colors();
  $img_empty =  ($exportpdf)
            ? drupal_get_path('module', 'isa_nifo') .'/images/isa_nifo_table_radar.png'
            : base_path() . drupal_get_path('module', 'isa_nifo') .'/images/isa_nifo_table_radar.png';
  $img_bottom =  ($exportpdf)
            ? drupal_get_path('module', 'isa_nifo') .'/images/bottom.png'
            : base_path() . drupal_get_path('module', 'isa_nifo') .'/images/bottom.png';
  $one_px = ($exportpdf)
            ? drupal_get_path('module', 'isa_nifo') .'/images/one_px.png'
            : base_path() . drupal_get_path('module', 'isa_nifo') .'/images/onepx.png';
  $width = '38';
  $border = 'border:1px solid #ccc;';
  $style_span0 = 'style=\'float:left;position:relative;left:-5px;\'';
  $style_span1 = 'style=\'float:left;position:relative;left:-9px;\'';
  $style_span2 = 'style=\'float:left;position:relative;left:-9px;\'';
  //Gris del scoring:
  $scores = array('0', '1', '2');
  $html = "<table>";
  $html .= "<tr >";
  $html .= "<td><img src='$img_empty' width='".$width."' height='15' alt='' /></td>";

  while ( list($id_category, $category) = each($categories) ) {
    $html .= "<td colspan='3' width='15%' class='principles'>" . $category['order'] . '. ' . $category['name'] . "</td>";
  }
  if($zone == 'alignement'){
    $txtZone = t('Alignment');
  } else {
    $txtZone = t(ucfirst($zone));
  }
  $html .= "</tr>";
  $html .= "<tr >";
  $html .= "<th class='type-scoring'></th>";
  $html .= "<th></th><th></th><th class='empty'><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th></th><th></th><th class='empty'><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th></th><th></th><th class='empty'><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th></th><th></th><th class='empty'><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th></th><th></th><th class='empty'><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "</tr>";

  $html .= "<tr>";
  $html .= "<th class='type-scoring' style='border:0px;'>EIF " . $txtZone . "</th>";
  $html .= "<th style='border:0px;text-align:left;'><span ".$style_span0.">0</span></th><th style='border:0px;text-align:left;'><span ".$style_span1.">1</span></th><th class='empty' style='text-align:left;border:0px;'><span ".$style_span2.">2</span><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th style='border:0px;text-align:left;'><span ".$style_span0.">0</span></th><th style='border:0px;text-align:left;'><span ".$style_span1.">1</span></th><th class='empty' style='border:0px;text-align:left;'><span ".$style_span2.">2</span><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th style='border:0px;text-align:left;'><span ".$style_span0.">0</span></th><th style='border:0px;text-align:left;'><span ".$style_span1.">1</span></th><th class='empty' style='border:0px;text-align:left;'><span ".$style_span2.">2</span><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th style='border:0px;text-align:left;'><span ".$style_span0.">0</span></th><th style='border:0px;text-align:left;'><span ".$style_span1.">1</span></th><th class='empty' style='border:0px;text-align:left;'><span ".$style_span2.">2</span><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "<th style='border:0px;text-align:left;'><span ".$style_span0.">0</span></th><th style='border:0px;text-align:left;'><span ".$style_span1.">1</span></th><th class='empty' style='border:0px;text-align:left;'><span ".$style_span2.">2</span><img src='$img_empty' width='15' height='15' alt='' /></th>";
  $html .= "</tr>";
  $sw = 0;
  if( is_array($data) and count($data) > 0 ){
    while (list($country, $values) = each($data)) {
      $html .= "<tr style='height:1px;'>";
      $html .= "<td style='height:1px;' colspan='16'></td>";
      $html .= "</tr>";
      $html .= "<tr>";
      $html .= "<td class='country' >" . $country . "</td>";
      foreach ($values as $value) {
        $colour = $colours[$sw]['hexadecimal'];
//echo "<hr><br> value: " . $value;
        foreach ($scores as $score) {
//echo "<br> score: " . $score;
          if( $value === '-' ){
            $wgrey = '0';
            $wcolour = ($width);
            $perCent = '10';
            $colour = '#fff';
          } elseif($value == $score){
            $wcolour = '0';
            $wgrey = ($width);
            $perCent = '0';
          } elseif( $value>$score ){
            $int = (strpos($value, '.')!==false)?substr($value, 0, (strpos($value, '.'))):$value;
            $rest = (strpos($value, '.')!==false)?substr($value, (strpos($value, '.')+1), 1):0;
//echo "<br> int: " . $int;
//echo "<br> rest: " . $rest;

            if( $int > $score ){
                $perCent = '10';
                $wcolour = ($width);
                $wgrey = '0';
            } else{
              $wcolour = number_format(((($width-2)*$rest)/10),0);
              $wgrey = (($width-2)-$wcolour);
              $perCent = $rest;
            }
          } else {
            $wgrey = ($width);
            $wcolour = '0';
            $perCent = '0';
          }
          $perCent = $perCent*10;
//echo "<br>wcolour: " . $wcolour;
//echo "<br>wgrey: " . $wgrey;
//echo "<br> percent: " . $perCent;
          /*if( $score == 2 ){
            $none = ' style="display:none;"';
          } else {
            $none = '';
          }*/
          if( $exportpdf ){
            if( $score != 2 ) {
              $html .= "<td style='width:".$width."px;height:15px !important;'>";
                if($wcolour>0){
                  $html .= "<span style='float:left;background-color:".$colour.";'><img style='width:".($wcolour-1)."px;height:15px;border:1px solid #ccc;' src='$img_empty' alt='' /></span>";
                }
                if($wgrey>0){
                  $html .= "<span style='float:left;background-color:#CCCCCC'><img style='width:".($wgrey-2)."px;height:15px;border:1px solid #ccc;' src='$img_empty' alt='' /></span>";
                }
              $html .= "</td>";
            }
          } else {
            if( $score != 2 ) {
              $html .= "<td><div style='background-color:#ccc;".$border."''><div style='background-color:" . $colour . ";width:".$perCent."%'><img src='$img_empty' alt='' width='".$width."' height='15' /></div></div></td>";
            }
          }
        }

        $html .= "<td class='empty'><img src='$img_empty' width='30' height='15' alt='' /></td>";
      }
      $sw++;
      $html .= "</tr>";
    }
  }
  $html .= "</table>";
  return $html;
}


function isa_nifo_analytical_summary($data, $categories_nifo, &$countries, $las_analytical_model) {
  $explanation = $data['zone'];
  //print_r($explanation);die;
  unset($data['zone']);
  while (list($zone, $data_zone) = each($data)) {
    $summary[$zone] = isa_nifo_analytical_summary_zone($data_zone, $categories_nifo, $countries, $las_analytical_model, $zone);
  }
  return $summary;
}

/*
* data_zone [category][element][country] => Array ( [nifo_score_alignement] => '' [nifo_evidence_alignement] => '')
* categories_nifo
* [categories] => array(id_category => Category)
* [elements] => Array([idElement] => Array(
*               [nifo_categories_idnifo_categories] => 1
*               [nifo_eif_element] => Subsidiarity and proportionality
*               [nifo_eif_measurement] => Does the NIF contain the 'subsidiarity and pr
*               [nifo_eif_reference] => EIF,Chapter 2, Underlying principles
*           )
* relation[id_category] = array(idElements)
*/

function isa_nifo_analytical_summary_zone($data_zone, $categories_nifo, &$countries, $las_analytical_model, $zone) {
  $categories = $categories_nifo['categories'];
  while (list($id_cat, $data_cat) = each($data_zone)) {
    $table = '';
    $sw = 1;
    //$head = $id_cat . ". " . $categories[$id_cat];
    // To name the categories is better concatenated the order instead the identificator
    $head = $categories[$id_cat]['order'] . ". " . $categories[$id_cat]['name'];
    $sw_countries = FALSE;
    // table identifier must be different to validate the page
    if($zone=='alignement'){
      $txtZone = t('Alignment');
    } else {
      $txtZone = t(ucfirst($zone));
    }
    $table .= '<table id="summary-' . $zone . '-' . $id_cat . '"><tr><td class="type-scoring">EIF '. $txtZone . " " . t('by Country') . '</td>';
    foreach (array_keys($las_analytical_model) as $country) {
      $table .= '<td class="country">' . $countries[$country] . '</td>';
    }
    $table .= '</tr>';
    $elements   = $categories_nifo['relation'][$id_cat];
    if (count($elements)) {
      foreach ($elements as $id_element) {
            //nifo_categories_idnifo_categories nifo_eif_element nifo_eif_measurement nifo_eif_reference /*
        $cat_name = "" != ($categories_nifo['elements'][$id_element]['nifo_eif_element']) ? $categories_nifo['elements'][$id_element]['nifo_eif_element'] : $categories_nifo['elements'][$id_element]['nifo_eif_measurement'];
        $class = $sw % 2 == 0 ? 'zebra' : '';
            // To name the categories is better concatenated the order instead the identificator
        $table .= '<tr><td class="categories ' . $class . '">' . $categories[$id_cat]['order'] . "." . $sw . " " . $cat_name . '</td>';
        $values = $data_zone[$id_cat][$id_element];
        reset($countries);
        foreach (array_keys($las_analytical_model) as $id_country) {
          $table .= "<td class='scoring $class'>";
          $table .= "<a href=\"".base_path()."nifo/analytical/evidence/" . $id_country . "/" . $id_cat . "/" . $id_element . "/" . $las_analytical_model[$id_country]['node']->nid . "/" . $zone . "\" class=\"enl-evidence\">";
          $table .= $values[$id_country]['nifo_score_' . $zone] != '' ? strval($values[$id_country]['nifo_score_' . $zone]) : "-";
          $table .= "</a>";
          $table .= "</td>";
        }
        $table .= '</tr>';
        $sw++;
      }
    }
    $table .= '</table>';
    $output['table'][$id_cat] = $table;
    $output['head'][$id_cat] = $head;
  }
  return $output;
}

function isa_nifo_get_taxonomy_term_name($tid) {
  $result = db_query('SELECT tid, name FROM {term_data} WHERE tid = %d', $tid);
  $term = db_fetch_array($result);
  if ($term) {
    return $term;
  }
}

function isa_nifo_get_category($id_nifo_category) {
  $result  = db_query("SELECT * FROM {nifo_categories} where idnifo_categories = %d", $id_nifo_category);
  $category = db_fetch_array($result);
  if ($category) {
    return $category;
  }
}

function isa_nifo_get_element($nid_element) {
  $result  = db_query("SELECT * FROM {nifo_eif_element} where idnifo_eif_element = %d", $nid_element);
  $element = (db_fetch_array($result));
  if ($element) {
    return $element;
  }
}

/*
check if the status is published and check if there are any items blank.
*/
function isa_nifo_workflow_published($node){
  if( variable_get('isa_nifo_workflow_published', '') == $node->workflow ){
    isa_nifo_check_elements_blank($node);
  }
}

/*
check if there are any items blank
*/
function isa_nifo_check_elements_blank($node){
  $vars = get_object_vars($node);
  $alignmentBlank   = 0;
  $monitoringBlank  = 0;
  $alignmentTotal   = 0;
  $monitoringTotal  = 0;
  $nifo_values = $vars['nifo_values'];
  if( is_array($nifo_values) and count($nifo_values) > 0 ){
    foreach($nifo_values as $vars) {
      $alignmentTotal++;
      if( $vars->nifo_score_alignement  == 'NULL' || $vars->nifo_score_alignement == '' ){
        $alignmentBlank++;
      }
      $monitoringTotal++;
      if( $vars->nifo_score_monitoring  == 'NULL' || $vars->nifo_score_monitoring == '' ){
        $monitoringBlank++;
      }
    }
  } elseif( count($vars) > 0 ){
    while( list($var, $val) = each($vars) ){
      // alignment-nifo_score_alignement-XX
      // monitoring-nifo_score_monitoring-XX
      if(strpos($var, 'alignment-nifo_score_alignement-') !== false){
        $alignmentTotal++;
        if( $val == 'NULL' || $val == '' ){
          $alignmentBlank++;
        }
      }

      if(strpos($var, 'monitoring-nifo_score_monitoring-') !== false){
        $monitoringTotal++;
        if( $val == 'NULL' || $val == '' ){
          $monitoringBlank++;
        }
      }
    }
  }
  if( $alignmentTotal > 0 || $monitoringTotal > 0 ){
    $percent = isa_nifo_percent($alignmentTotal+$monitoringTotal, $alignmentBlank+$monitoringBlank);
    if( $percent > 0 ){
      $percent = number_format($percent, 2);
      drupal_set_message(t('You are posting an analytical model with %percent % of unfilled score', array('%percent' => $percent)), "warning");
    }
  }

}

function isa_nifo_percent($total, $blank){
  return ($blank*100)/$total;
}


