<?php

/**
 * @file
 *
 * @ingroup isa_node_form
 *
 */

/**
 *
 * Validation function for asset_release_form
 *
 * @param array $form An associative array containing the structure of the form.
 * @param array $form_state A keyed array containing the current state of the form.
 *
 * @see isa_node_form_form_alter
 *
 */
function _asset_release_form_validate(&$form, &$form_state) {
  //Validate the translations
  isa_node_form_validate_translations($form_state, 'field_asset_name', 'name', TRUE);
  isa_node_form_validate_translations($form_state, 'field_asset_alternative_name', 'alternative name', FALSE);
  isa_node_form_validate_translations($form_state, 'field_asset_description', 'description', TRUE);

  //Validate the distribution(cardinality [0:1] to asset release
  isa_node_form_validate_distribution($form_state, 'field_asset_distribution', 'distribution');
  $release_taxonomy = _asset_release_taxonomies_by_type();
  //Validation for asset releases
  if ('asset' == $form_state['values']['field_asset_release_type'][0]['value']) {
    //Reattach taxonomies to the correct form data structure to save them
    array_walk($release_taxonomy, function($val, $key) use(&$form_state){
      if('asset' == $val) {
        //$form_state['clicked_button']['#post']['taxonomy'][$key] = $form_state['values']['taxonomy'][$key] = $form_state['values'][$key];
        //??
      }
    });
    //Asset type is required for asset Releases
    //$form_state['values'][variable_get('status_vid', 69)] > 0 ? TRUE : form_error($form['group_asset_asset']['taxonomy'][variable_get('status_vid', 69)], t('Field Status is required'));
    $error = TRUE;
    foreach ($form_state['values'][variable_get('asset_type_vid', 68)] as $key => $value) {
      if ($value > 0) {
        $error = FALSE;
      }
    }
    //Asset status is required for asset Releases
    $error ? form_error($form['group_asset_asset']['taxonomy'][variable_get('asset_type_vid', 68)], t('Field Solution Type is required')) : FALSE;
  }
}
/**
 * Form alter for the asset release form
 *
 * @param array $form Nested array of form elements that comprise the form.
 * @param array $form_state A keyed array containing the current state of the form. The arguments that drupal_get_form() was originally called with are available in the array $form_state['build_info']['args'].
 * @param string $form_id String representing the name of the form itself. Typically this is the name of the function that generated the form.
 *
 * @see isa_node_form_form_alter
 */
function _asset_release_form_alter(&$form, &$form_state, $form_id) {
  $form['#after_build'] = array('_asset_release_js_form');

  //Get type of release based on software parent or pop up reference
  $node = isset($form['#node']) ? $form['#node'] : FALSE;
  $path = isset($_GET['destination']) ? $_GET['destination'] : '';

  if (isset($_GET['release_type'])) {
    $type = $_GET['release_type'];
  } else {
    if (!$tempType = $form['group_is_mandatory_information']['field_asset_release_type']['#default_value'][0]['value']) {
      $tempType = $form['group_is_mandatory_information']['field_asset_release_type']['#value'][0]['value'];
    }
    $type = _release_is_asset_or_software($tempType, $node, $path);
  }

  _asset_release_set_type_fields($form, $type);
  
  //Required FALSE fields
  //Set these to not required and check it in _asset_release_form_validate(), depending on the value of Release Type
  $form['taxonomy'][variable_get('asset_type_vid', 68)]['#required'] = FALSE;
  $form['taxonomy'][variable_get('status_vid', 69)]['#required'] = FALSE;
  $form['taxonomy'][variable_get('geographic_coverage_vid', 26)]['#required'] = FALSE;
  $form['taxonomy'][variable_get('domains_vid', 27)]['#required'] = FALSE;
  
  // Required TRUE fields
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-653
  // removed title for field asset_node_reference as requested by PwC (it is enough with the text in the "grey bar" above)
  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1058
  // Adapt to EFIR specifications: Description and Themes to required
  $form['taxonomy'][variable_get('domains_vid', 27)]['#required'] = TRUE;
  $form ['body_field']['body']['#required'] = TRUE;

  //Set the publiser as required
  $form['group_is_mandatory_information']['field_asset_publisher']['#required'] = TRUE;
  $form['group_is_mandatory_information']['field_asset_publisher'][0]['#required'] = TRUE;

  isa_node_form_alter_og_audience_form($form, $form_state, $form_id);

  unset($form['field_asset_last_modification']);

  // Add checkbox around field_asset_identifier
  $node = $form['#node'];
  $field_asset_identifier_value = !empty($node->field_asset_identifier[0]['value']);
  $field_asset_identifier_weight = $form['group_is_additional_information']['field_asset_identifier']['#weight'] - 0.2;
  $form['group_is_additional_information']['field_asset_identifier_chk'] = array(
    '#type' => 'checkbox',
    '#title' => t('Do you have an identifier?'),
    '#default_value' => $field_asset_identifier_value,
    '#weight' => $field_asset_identifier_weight,
  );

  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-733
  // contributor agreement for contributed metadata ("Propose your federated repository" and "Propose your asset release", edit mode)
  $end_text = variable_get('group_propose_contributed_metadata_agreement_information', FALSE);
  if ($end_text) {
    $form['buttons']['#prefix'] = token_replace($end_text, 'global');
  }

  // https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1096
  // make the field unaccessible for the user
  // if we get the nid by url we put the value FORMAT [nid:NID]
  if (isset($_GET['gids'])) {
    //When passing gids value, disable field
    $form['group_prs_reference']['field_prs_reference'][0]['#value']['nid'] = '[nid:' . $_GET['gids'][0] . ']';
    $form['group_prs_reference']['field_prs_reference'][0]['#access'] = FALSE; // if we click on any "propouse your asset release" button we dont should to be able to change this value, have no sense.
  } elseif (isset($form['#node']->og_groups) && !empty($form['#node']->og_groups) && empty($form['group_prs_reference']['field_prs_reference'][0]['#default_value']['nid'])) {
    $group = node_load(key($form['#node']->og_groups));
    $group_reference = $group->title . " [nid:" . $group->nid . "]";
    $form['group_prs_reference']['field_prs_reference'][0]['#value']['nid'] = $group_reference;
  }
  
  //hide asset release type field
  $form['group_is_mandatory_information']['field_asset_release_type']['#access'] = FALSE;
}

function _asset_release_taxonomies_by_type() {
  return array(
      variable_get('asset_type_vid', 68) => 'asset',//Asset type
      variable_get('status_vid', 69) => 'asset',//Status
      variable_get('interoperability_level_vid',74) => 'asset',//Interoperability level
      variable_get('trove_status_vid',82) => 'software', //Trove: status

    );
}
function _asset_release_fields_by_type() {
  return array(
    'asset' => array(
      'field_asset_temporal_coverage',
      'field_asset_item',
      'field_asset_homepage_doc',
      'field_asset_node_reference',
    ),
    'software' => array(
      'field_asset_sw_logo',
      'field_asset_sw_report',
      'field_asset_sw_usedby',
      'field_asset_sw_metrics',
      'field_asset_sw_release_type',
      ),
    );
}
function _asset_release_js_form($form, &$form_state) {
  if(true == module_exists('isa_jquery_noconflict')){
      isa_jquery_noconflict_include_jquery();
      $path = drupal_get_path('module', 'isa_node_form');
      drupal_add_js($path .'/js/node_form_asset_release.js');
  }
  
  //move no mandatory taxonomy to another field 
  $form['not_mandatory_taxonomy'] = $form['taxonomy'];
  foreach ($form['taxonomy'] as $k=>$t) {
    if (!$t['#required']) {
      //unset field from mandatory taxonomy list
      unset($form['taxonomy'][$k]);
    } else {
      //unset field from not mandatory taxonomy list
      unset($form['not_mandatory_taxonomy'][$k]);
    }
  }
  
  //move title and description node fields into mandatory information group
  isa_node_form_move_fields_fieldgroup($form, 'group_is_mandatory_information', array('title', 'body_field', 'taxonomy'));
  //order fields in mandatory information group 
  isa_node_form_order_fields_fieldgroup($form, 'group_is_mandatory_information', array(
      'field_id_uri',
      'title',
      'body_field',
      'field_asset_release_type',
      'taxonomy',
      'field_asset_publisher'
  ));
  
  //add not_mandatory_taxonomy to group_is_additional_information
  isa_node_form_move_fields_fieldgroup($form, 'group_is_additional_information', array('not_mandatory_taxonomy'));
  
  //order fields in group_is_additional_information
  isa_node_form_order_fields_fieldgroup($form, 'group_is_additional_information', array(
      1000 => 'not_mandatory_taxonomy'
  ));
  
  //collaps field groups 
  isa_node_form_collaps_fieldgroup($form, array(
      'group_is_additional_information',
      'revision_information',
      'group_prs_reference',
      'options',
      'workflow',
      'og_nodeapi'
  ));
  
  return $form;
}
/**
 * Returns the release type from the release to which the distribution is associated.
 *
 */
function _release_is_asset_or_software($current_type, $software_project, $sw_destination) {
  //On edition type can`t be change
  if ('edit' == arg(2)) {
    return $current_type;
  }
  //New release from a software project by pop up
  $sw_destination = explode("/", $sw_destination);
  if ('add' == $sw_destination[1] and 'project' == $sw_destination[2]) {
    return 'software';
  }
  //Check if it this release is associated to a software project using pop up form
  $node_sw = node_load($sw_destination[1]);
  if ('project' == $node_sw->type) {
    return 'software';
  }
  return '';
}
function _asset_release_set_type_fields(&$form, $type){
  //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1058
  //Logo and metrics are not shown in the “propose your semantic asset” GUI.
  if ('software' == $type) {
    $form['group_is_mandatory_information']['field_asset_release_type']['#default_value'][0]['value'] = 'software';
    $form['group_is_mandatory_information']['field_asset_release_type']['#value'][0]['value'] = 'software';
    $form['group_is_mandatory_information']['field_asset_release_type']['#disabled'] = TRUE;
  }
  if ('asset' == $type) {
    $form['group_is_mandatory_information']['field_asset_release_type']['#default_value'][0]['value'] = 'asset';
    $form['group_is_mandatory_information']['field_asset_release_type']['#value'][0]['value'] = 'asset';
    $form['group_is_mandatory_information']['field_asset_release_type']['#disabled'] = TRUE;
  }
}