<?php

/**
 * @file
 *
 * @ingroup isa_node_form
 *
 */

/**
 *
 * Validation function for distribution_form
 *
 * @param array $form An associative array containing the structure of the form.
 * @param array $form_state A keyed array containing the current state of the form.
 *
 * @see isa_node_form_form_alter
 *
 */
function _distribution_form_validate(&$form, &$form_state) {
  $isset_file = isset($form_state['values']['field_distribution_access_url'][0]) && $form_state['values']['field_distribution_access_url'][0]['fid'] != 0;
  $isset_url = isset($form_state['values']['field_distribution_access_url1'][0]) && !empty($form_state['values']['field_distribution_access_url1'][0]['url']);
  //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-658
// Only save one field: url or file
  //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1018
  //When trying to edit a distribution the error "Access URL or file field is required. Please set an URL or upload a file" keeps coming back.
  $type = $form_state['clicked_button']['#post']['group1_dist'];
  if ($isset_url) {
    if ($isset_url) {
      foreach ($form_state['values']['field_distribution_access_url1'] as $url) {
        if (!is_array($url) || empty($url['url'])) {
          continue;
        }
        $url_check = drupal_http_request($url['url']);
        //correspond to RFC 1738
        if ($url_check->code < 0 || !link_validate_url($url['url'])) {
          form_set_error('field_distribution_access_url1', t('The url (%url) is not valid (not formatted according to RFC 1738).', array('%url' => $url['url'])));
        }
        //don't return a 4xx or 5xx error
        if (($url_check->code >= 400 && $url_check->code <= 600)) {
          form_set_error('field_distribution_access_url1', t('The url %url is not valid (return a %code error).', array('%url' => $url['url'], '%code' => $url_check->code)));
        }
      }
      if ($form_state['values']['field_distribution_access_url'][0]['fid'] != 0) {
        $removed_file = field_file_load($form_state['values']['field_distribution_access_url'][0]['fid']);
        $form_state['values']['field_distribution_access_url'][0]['fid'] = 0;
        field_file_delete($removed_file);
      }
    }
  }
  //If there is no URL filled, we check if a file was uploaded
  else {
    if (!$isset_file) {
      form_set_error('field_distribution_access_url', t('Access URL is required in case no Access file is provided. Please provide the Access URL or upload a file.'));
      form_set_error('field_distribution_access_url1', t('Access file field is required in case no Access URL was provided. Please upload a file or provide the Access URL.'));
    } else {
      $form_state['values']['field_distribution_access_url1'][0] = "";
    }
  }
  //Validate the translations
  isa_node_form_validate_translations($form_state, 'field_distribution_name', 'name', TRUE);
  isa_node_form_validate_translations($form_state, 'field_distribution_description', 'description', TRUE);

  //set a message when one or more selected licence is not validated
  $warning_licence = 0;

  foreach ($form_state['values']['field_distribution_licence'] as $licence) {
    if (is_array($licence)) {
      if (isset($licence['nid'])) {
        $node = node_load($licence['nid']);
        if (workflow_node_current_state($node) != ISA_SID_COMMUNITY_VALIDATED) {
          //if the publisher is not in validated state
          $warning_licence ++;
        }
      }
    }
  }
  //set a message if one or more licence is not validated
  if ($warning_licence > 1) {
    drupal_set_message('several licences should be approved by the moderators');
  } elseif ($warning_licence > 0) {
    drupal_set_message('one licence should be approved by the moderators');
  }

  //set a message when one or more selected publisher is not validated
  $warning_publisher = 0;

  foreach ($form_state['values']['field_distribution_publisher'] as $publisher) {
    if (is_array($publisher)) {
      if (isset($publisher['nid'])) {
        $node = node_load($publisher['nid']);
        if (workflow_node_current_state($node) != ISA_SID_COMMUNITY_VALIDATED) {
          //if the publisher is not in validated state
          $warning_publisher ++;
        }
      }
    }
  }
  //set a message if one or more publisher is not validated
  if ($warning_publisher > 1) {
    drupal_set_message('several publishers should be approved by the moderators');
  } elseif ($warning_publisher > 0) {
    drupal_set_message('one publisher should be approved by the moderators');
  }
}

/**
 * Form alter for the distribution form
 *
 * @param array $form Nested array of form elements that comprise the form.
 * @param array $form_state A keyed array containing the current state of the form. The arguments that drupal_get_form() was originally called with are available in the array $form_state['build_info']['args'].
 * @param string $form_id String representing the name of the form itself. Typically this is the name of the function that generated the form.
 *
 * @see isa_node_form_form_alter
 */
function _distribution_form_alter(&$form, &$form_state, $form_id) {

  $node = isset($form['#node']) ? $form['#node'] : FALSE;
  $parse_release_nid = isset($_GET['destination']) ? explode("/", $_GET['destination']) : FALSE;
  unset($form['field_asset_last_modification']);

  //Add the title on child of node reference (for the error message)
  $form['group_dis_additional_information']['field_distribution_publisher'][0]['#title'] = $form['group_dis_additional_information']['field_distribution_publisher']['#title'];

  $form['group_dis_mandatory_information']['field_distribution_licence'][0]['#title'] = $form['group_dis_mandatory_information']['field_distribution_licence']['#title'];

  //Hide file format field
  $form['taxonomy'][73]['#access'] = false;

  // Add custom form_submit handler
  array_unshift($form['#submit'], '_distribution_form_submit_handler');

  // Add custom after build handler
  $form['#after_build'] = array('_distribution_form_after_build');
}

/**
 * Order and add form fields in specific groups
 * for distribution content type
 * 
 * @param array $form
 * @param array $form_state
 * @return array $form
 */
function _distribution_form_after_build(&$form, $form_state) {
  // add title field to mandatory info group 
  isa_node_form_move_fields_fieldgroup($form, 'group_dis_mandatory_information', array('title'));
  // order fields inside mandatory info group 
  isa_node_form_order_fields_fieldgroup($form, 'group_dis_mandatory_information', array(
      'field_id_uri',
      'title',
      'field_distribution_licence',
      'field_distribution_access_url1',
      'field_distribution_access_url'
  ));

  // add description and taxonomy fields to additional info group 
  isa_node_form_move_fields_fieldgroup($form, 'group_dis_additional_information', array('body_field', 'taxonomy'));
  // order fields inside additional info group 
  isa_node_form_order_fields_fieldgroup($form, 'group_dis_additional_information', array(
      'body_field',
      'field_distribution_description',
      'field_distribution_name',
      'field_distribution_sw_tagurl',
      'taxonomy',
      'field_distribution_sw_identifier',
      'field_distribution_sw_checksum',
      'field_distribution_sw_filesizein',
      'field_distribution_download_url',
      'field_distribution_publisher'
  ));

  // collaps options and additional info groups
  isa_node_form_collaps_fieldgroup($form, array(
      'group_dis_additional_information',
      'options'
  ));

  return $form;
}

/**
 * Form submit handler for 'distribution' nodes
 * @param type $form
 * @param type $form_state
 */
function _distribution_form_submit_handler($form, &$form_state) {

  if (isset($form_state['values']['field_distribution_access_url1'][0]) && !empty($form_state['values']['field_distribution_access_url1'][0])) {
    $url = parse_url($form_state['values']['field_distribution_access_url1'][0]['url']);
    $ext = explode('.', $url['path']);
    $ext = end($ext);
    $terms = taxonomy_get_term_by_name($ext);
    if ($term) {
      $tid = $terms[0]->tid;
    }
    if (!$tid) {
      $result = db_query("SELECT tid FROM {term_fields_term} WHERE taxonomy_uri_value LIKE '%" . $ext . "' OR taxonomy_legacy_uri_value LIKE '%" . $ext . "'");
      $row = db_fetch_array($result);
      if ($row) {
        $tid = $row['tid'];
      }
    }
  } elseif (isset($form_state['values']['field_distribution_access_url'][0]) && !empty($form_state['values']['field_distribution_access_url'][0])) {
    $filemime = $form_state['values']['field_distribution_access_url'][0]['filemime'];

    $terms = taxonomy_get_term_by_name($filemime);
    if ($term) {
      $tid = $terms[0]->tid;
    }
    if (!$tid) {
      $result = db_query("SELECT tid FROM {term_fields_term} WHERE taxonomy_uri_value LIKE '%" . $filemime . "' OR taxonomy_legacy_uri_value LIKE '%" . $filemime . "'");
      $row = db_fetch_array($result);
      if ($row) {
        $tid = $row['tid'];
      }
    }
  }
  if ($tid) {
    $form_state['values']['taxonomy'][73] = $tid;
  } else {
    $form_state['values']['taxonomy'][73] = NULL;
  }
}

/**
 * Returns the release type from the release to which the distribution is associated.
 * 
 */
function _distribution_is_asset_or_software($distribution, $release_destination) {
  //Check if it this distribution is associated with a release and use it to get it's type
  $sql = 'SELECT node.nid FROM {node}, {content_field_asset_distribution}
        WHERE type="asset_release"
        AND content_field_asset_distribution.vid=node.vid
        AND content_field_asset_distribution.field_asset_distribution_nid=%d';
  $release = db_result(db_query($sql, $distribution->nid));
  if ($release > 0) {
    $release = node_load($release);
  } else {
    //Check if it this distribution is associated to a release using pop up form
    $node_release = node_load($release_destination);
    $release = 'asset_release' == $node_release->type ? $node_release : '';
  }
  if (isset($release->field_asset_release_type[0]['value'])) {
    return $release->field_asset_release_type[0]['value'];
  } else {
    return '';
  }
}