<?php

require_once(drupal_get_path('module', 'node') . '/node.module');

/**
 * Form alter for the project_project form
 *
 * @param array $form Nested array of form elements that comprise the form.
 * @param array $form_state A keyed array containing the current state of the form. The arguments that drupal_get_form() was originally called with are available in the array $form_state['build_info']['args'].
 * @param string $form_id String representing the name of the form itself. Typically this is the name of the function that generated the form.
 *
 * @see isa_node_form_form_alter
 */
function _project_project_form_alter(&$form, &$form_state, $form_id) {
  //Transform the single Register a Project link into two differen links, for Asset projects and Software Projects
  drupal_add_js(drupal_get_path('module', 'isa_node_form') . '/js/node_form_project_project.js');
  //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1090
  //Order of the fields
  /* FIELDS FROM project  */
  // Hide 'field_project_adms_sw_project' field
  $form['field_project_adms_sw_project']['#access'] = FALSE;

  // field_project_common_type
  $form['field_project_common_type']['#weight'] = -1000;

  // FIELD_SW_PROJECT_METRICS
  $field_sw_project_metrics = content_fields('field_sw_project_metrics', 'project');
  $form['project']['field_sw_project_metrics'] = content_field_form($form, $form_state, $field_sw_project_metrics);

  // FIELD_SW_PROJECT_BUG_DATABASE
  $field_sw_project_bug_database = content_fields('field_sw_project_bug_database', 'project');
  $form['project']['field_sw_project_bug_database'] = content_field_form($form, $form_state, $field_sw_project_bug_database);

  // FIELD_SW_PROJECT_WIKI
  $field_sw_project_wiki = content_fields('field_sw_project_wiki', 'project');
  $form['project']['field_sw_project_wiki'] = content_field_form($form, $form_state, $field_sw_project_wiki);

  // FIELD_SW_PROJECT_REPOSITORY
  $field_sw_project_repository = content_fields('field_sw_project_repository', 'project');
  $form['project']['field_sw_project_repository'] = content_field_form($form, $form_state, $field_sw_project_repository);

  // FIELD_SW_PROJECT_AGENTS
  $field_sw_project_agents = content_fields('field_sw_project_agents', 'project');
  $form['project']['field_sw_project_agents'] = content_field_form($form, $form_state, $field_sw_project_agents);

  // FIELD_SW_RELEASE
  $field_sw_release = content_fields('field_sw_release', 'project');
  $form['project']['field_sw_release'] = content_field_form($form, $form_state, $field_sw_release);




  unset($form['taxonomy'][32]);
  unset($form['taxonomy'][31]);
  unset($form['taxonomy'][38]);
  unset($form['taxonomy'][34]);
  unset($form['taxonomy'][30]);

  // FIELD FOR TOPIC TAXONOMY
  $field_topic = taxonomy_form(80, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_topic'] = $field_topic;

  // FIELD FOR USER_INTERFACE TAXONOMY
  $field_user_interface = taxonomy_form(81, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_user_interface'] = $field_user_interface;

  // FIELD FOR PROGRAMMING_LANGUAGE TAXONOMY
  $field_programming_language = taxonomy_form(79, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_programming_language'] = $field_programming_language;

  // FIELD FOR OPERATING_SYSTEM TAXONOMY
  $field_operating_system = taxonomy_form(83, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_operating_system'] = $field_operating_system;

  // FIELD FOR INTENDED_AUDIENCE TAXONOMY
  $field_intended_audience = taxonomy_form(78, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_intended_audience'] = $field_intended_audience;

   // FIELD FOR LOCALE TAXONOMY
  $field_locale = taxonomy_form(77, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_locale'] = $field_locale;

   // FIELD FOR STATUS TAXONOMY
  $field_status = taxonomy_form(82, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_status'] = $field_status;

  // FIELD FOR FILE FORMAT
  $field_file_format = taxonomy_form(73, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_file_format'] = $field_file_format;

  // FIELD FOR SOLUTION CATEGORY
  $field_solution_category = taxonomy_form(84, 0, NULL, 'taxonomy');
  $form['taxonomy']['field_solution_category'] = $field_solution_category;

  // When edit a node 'project'
  if (isset($form['#node']->field_project_adms_sw_project[0]['nid'])) {
    $node = node_load($form['#node']->field_project_adms_sw_project[0]['nid']);

    // FIELD_SW_PROJECT_METRICS
    $i = 0;
    $value_type = $form['project']['field_sw_project_metrics']['field_sw_project_metrics']['0'];
    while (isset($node->field_sw_project_metrics[$i]['nid'])) {
      $form['project']['field_sw_project_metrics']['field_sw_project_metrics'][$i] = $value_type;
      $form['project']['field_sw_project_metrics']['field_sw_project_metrics'][$i]['#default_value']['nid'] = $node->field_sw_project_metrics[$i]['nid'];
      $i++;
    }

    // FIELD_SW_PROJECT_BUG_DATABASE
    $i = 0;
    $value_type = $form['project']['field_sw_project_bug_database']['field_sw_project_bug_database']['0'];
    while (isset($node->field_sw_project_bug_database[$i]['value'])) {
      $form['project']['field_sw_project_bug_database']['field_sw_project_bug_database'][$i] = $value_type;
      $form['project']['field_sw_project_bug_database']['field_sw_project_bug_database'][$i]['#default_value']['value'] = $node->field_sw_project_bug_database[$i]['value'];
      $i++;
    }

    // FIELD_SW_PROJECT_WIKI
    $i = 0;
    $value_type = $form['project']['field_sw_project_wiki']['field_sw_project_wiki']['0'];
    while (isset($node->field_sw_project_wiki[$i]['value'])) {
      $form['project']['field_sw_project_wiki']['field_sw_project_wiki'][$i] = $value_type;
      $form['project']['field_sw_project_wiki']['field_sw_project_wiki'][$i]['#default_value']['value'] = $node->field_sw_project_wiki[$i]['value'];
      $i++;
    }

    // FIELD_SW_PROJECT_REPOSITORY
    $i = 0;
    $value_type = $form['project']['field_sw_project_repository']['field_sw_project_repository']['0'];
    while (isset($node->field_sw_project_repository[$i]['nid'])) {
      $form['project']['field_sw_project_repository']['field_sw_project_repository'][$i] = $value_type;
      $form['project']['field_sw_project_repository']['field_sw_project_repository'][$i]['#default_value']['nid'] = $node->field_sw_project_repository[$i]['nid'];
      $i++;
    }

	  // FIELD_SW_PROJECT_AGENTS
    $i = 0;
    $value_type = $form['project']['field_sw_project_agents']['field_sw_project_agents']['0'];
    while (isset($node->field_sw_project_agents[$i]['nid'])) {
      $form['project']['field_sw_project_agents']['field_sw_project_agents'][$i] = $value_type;
      $form['project']['field_sw_project_agents']['field_sw_project_agents'][$i]['#default_value']['nid'] = $node->field_sw_project_agents[$i]['nid'];
      $i++;
    }

	  //FIELD_SW_RELEASE
	  $i = 0;
    $value_type = $form['project']['field_sw_release']['field_sw_release']['0'];
    while (isset($node->field_sw_release[$i]['nid'])) {
      $form['project']['field_sw_release']['field_sw_release'][$i] = $value_type;
      $form['project']['field_sw_release']['field_sw_release'][$i]['#default_value']['nid'] = $node->field_sw_release[$i]['nid'];
      $i++;
    }

    // FIELD FOR TOPIC TAXONOMY and USER_INTERFACE TAXONOMY 
    $array_topic = array();
    $array_user_interface = array();
	  $array_programming_language = array();
	  $array_operating_system = array();
	  $array_intended_audience = array();
	  $array_locale = array();
	  $array_status = array();
	  $array_file_format = array();
	  $array_solution_category = array();
    foreach ($node->taxonomy as $term) {
      switch ($term->vid) {
        // field_topic
        case 80:
          array_push($array_topic, $term->tid);
          break;
        // field_user_inerface
        case 81:
          array_push($array_user_interface, $term->tid);
          break;
        // field_programming_language
        case 79:
          array_push($array_programming_language, $term->tid);
          break;
        // field_operating_system
        case 83:
          array_push($array_operating_system, $term->tid);
          break;
        // field_intended_audience
        case 78:
          array_push($array_intended_audience, $term->tid);
          break;
        // field_locale
        case 77:
          array_push($array_locale, $term->tid);
          break;
        // field_status
        case 82:
          array_push($array_status, $term->tid);
          break;
        // field_file_format
        case 73:
          array_push($array_file_format, $term->tid);
          break;
        // field_solution_category
        case 84:
          array_push($array_solution_category, $term->tid);
          break;
      }
    }
    $form['taxonomy']['field_topic']['#default_value'] = $array_topic;
    $form['taxonomy']['field_user_interface']['#default_value'] = $array_user_interface;
    $form['taxonomy']['field_operating_system']['#default_value'] = $array_operating_system;
    $form['taxonomy']['field_programming_language']['#default_value'] = $array_programming_language;
    $form['taxonomy']['field_intended_audience']['#default_value'] = $array_intended_audience;
    $form['taxonomy']['field_locale']['#default_value'] = $array_locale;
    $form['taxonomy']['field_status']['#default_value'] = $array_status;
    $form['taxonomy']['field_file_format']['#default_value'] = $array_file_format;
    $form['taxonomy']['field_solution_category']['#default_value'] = $array_solution_category;
  }

  // Add custom form_submit handlet
  array_unshift($form['#submit'], '_project_project_form_submit_handler');
}

/**
 * Form submit handler for 'project-project' nodes
 * @param type $form
 * @param type $form_state
 */
function _project_project_form_submit_handler($form, &$form_state) {

  // If 'field_project_adms_sw_project' is not null, it is an update. If not, it is a new insert.
  if (isset($form_state['values']['field_project_adms_sw_project']['0']['nid'])) {
    // Load 'project' node to update
    $node = node_load($form_state['values']['field_project_adms_sw_project']['0']['nid']);
  } else{
    $node = new StdClass();
    $node->type = 'project';
	global $user;
    $node->uid = $user->uid;
  }

  //Body
  $node->body = $form_state['values']['body'];
  // Language
  $node->language = $form_state['values']['language'];
  // 'project' title
  $node->title = 'SW - ' . $form_state['values']['title'];
  // Published
  $node->status = 1;
  // Workflow: Validated
  $node->_workflow = 61;

  
  // Fill 'project' node fields

  $i = 0;
  while (isset($form_state['values']['field_id_uri'][$i]['value'])) {
    $node->field_id_uri[$i]['value'] = $form_state['values']['field_id_uri'][$i]['value'];
    $i++;
  }
  
  $i = 0;
  $node->field_sw_project_metrics = array();
  while (isset($form_state['values']['project']['field_sw_project_metrics']['field_sw_project_metrics'][$i]['nid'])) {
    $node->field_sw_project_metrics[$i]['nid'] = $form_state['values']['project']['field_sw_project_metrics']['field_sw_project_metrics'][$i]['nid'];
    $i++;
  }
  
  $i = 0;
  $node->field_sw_project_bug_database = array();
  while (isset($form_state['values']['project']['field_sw_project_bug_database']['field_sw_project_bug_database'][$i]['value'])) {
    $node->field_sw_project_bug_database[$i]['value'] = $form_state['values']['project']['field_sw_project_bug_database']['field_sw_project_bug_database'][$i]['value'];
    $i++;
  }
  
  $i = 0;
  $node->field_sw_project_wiki = array();
  while (isset($form_state['values']['project']['field_sw_project_wiki']['field_sw_project_wiki'][$i]['value'])) {
    $node->field_sw_project_wiki[$i]['value'] = $form_state['values']['project']['field_sw_project_wiki']['field_sw_project_wiki'][$i]['value'];
    $i++;
  }
  
  $i = 0;
  $node->field_sw_project_repository = array();
  while (isset($form_state['values']['project']['field_sw_project_repository']['field_sw_project_repository'][$i]['nid'])) {
    $node->field_sw_project_repository[$i]['nid'] = $form_state['values']['project']['field_sw_project_repository']['field_sw_project_repository'][$i]['nid'];
    $i++;
  }


   $i = 0;
  $node->field_sw_project_agents = array();
  while (isset($form_state['values']['project']['field_sw_project_agents']['field_sw_project_agents'][$i]['nid'])) {
    $node->field_sw_project_agents[$i]['nid'] = $form_state['values']['project']['field_sw_project_agents']['field_sw_project_agents'][$i]['nid'];
    $i++;
  }
  
   $i = 0;
  $node->field_sw_release = array();
  while (isset($form_state['values']['project']['field_sw_release']['field_sw_release'][$i]['nid'])) {
    $node->field_sw_release[$i]['nid'] = $form_state['values']['project']['field_sw_release']['field_sw_release'][$i]['nid'];
    $i++;
  }

  //lets save taxonomies fields
  $taxonomies = array(
    36,
    27,
    'field_topic',
    'field_user_interface',
    'field_programming_language',
    'field_operating_system',
    'field_intended_audience',
    'field_locale',
    'field_status',
    'field_file_format',
    'field_solution_category',
  );
  foreach ($taxonomies  as $tax){
    if (isset($form_state['values'][$tax])){
      $form_state['values']['taxonomy'][$tax] = (is_numeric($tax)) ? array_filter($form_state['values'][$tax]) :
        $form_state['values'][$tax];
    }
  }

  //tags
  $form_state['values']['taxonomy']['tags'][28] = $form_state['values'][28];
  //I.solution
  $form_state['values']['taxonomy'][43] = $form_state['values'][43];
  $node->taxonomy= array();
 //Save software taxonomies
  foreach($form_state['values']['taxonomy']['field_topic'] as $key=>$tid){
    if($tid){
	  $node->taxonomy[$tid]= taxonomy_get_term($tid);
	  $form_state['values']['taxonomy']['field_topic'][$key]=0;
	}
  }
  foreach($form_state['values']['taxonomy']['field_user_interface'] as $key=>$tid){
    if($tid){
	  $node->taxonomy[$tid]= taxonomy_get_term($tid);
	  $form_state['values']['taxonomy']['field_user_interface'][$key]=0;
	}
  }
  foreach($form_state['values']['taxonomy']['field_operating_system'] as $key=>$tid){
    if($tid){
	  $node->taxonomy[$tid]= taxonomy_get_term($tid);
	  $form_state['values']['taxonomy']['field_operating_system'][$key]=0;
	}
  }
  foreach($form_state['values']['taxonomy']['field_programming_language'] as $key=>$tid){
    if($tid){
	  $node->taxonomy[$tid]= taxonomy_get_term($tid);
	  $form_state['values']['taxonomy']['field_programming_language'][$key]=0;
	}
  }
  foreach($form_state['values']['taxonomy']['field_intended_audience'] as $key=>$tid){
    if($tid){
	  $node->taxonomy[$tid]= taxonomy_get_term($tid);
	  $form_state['values']['taxonomy']['field_intended_audience'][$key]=0;
	}
  }
  
  foreach($form_state['values']['taxonomy']['field_locale'] as $key=>$tid){
    if($tid){
	  $node->taxonomy[$tid]= taxonomy_get_term($tid);
	  $form_state['values']['taxonomy']['field_locale'][$key]=0;
	}
  }

  if(isset($form_state['values']['taxonomy']['field_file_format'])&&!empty($form_state['values']['taxonomy']['field_file_format'])){
	  $node->taxonomy[$form_state['values']['taxonomy']['field_file_format']]= taxonomy_get_term($form_state['values']['taxonomy']['field_file_format']);
	  $form_state['values']['taxonomy']['field_file_format']=array();
  }
  if(isset($form_state['values']['taxonomy']['field_status'])&&!empty($form_state['values']['taxonomy']['field_status'])){
	  $node->taxonomy[$form_state['values']['taxonomy']['field_status']]= taxonomy_get_term($form_state['values']['taxonomy']['field_status']);
	  $form_state['values']['taxonomy']['field_status']=array();
  }
  if(isset($form_state['values']['taxonomy']['field_solution_category'])&&!empty($form_state['values']['taxonomy']['field_solution_category'])){
	  $node->taxonomy[$form_state['values']['taxonomy']['field_solution_category']]= taxonomy_get_term($form_state['values']['taxonomy']['field_solution_category']);
	  $form_state['values']['taxonomy']['field_solution_category']=array();
  }

  
    if(isset($form_state['values']['field_project_soft_logo'][0]['fid'])&&!empty($form_state['values']['field_project_soft_logo'][0]['fid'])){
	  //There is already a software logo stored, edit it
	  if (isset($node->field_sw_project_logo[0])&&!empty($node->field_sw_project_logo[0])){
	    $logo = _isa_node_form_create_logo($form_state['values']['field_project_soft_logo'][0],$node->field_sw_project_logo[0]['nid']);
	  }else{
	    //There is no software logo stored, create it
	    $logo = _isa_node_form_create_logo($form_state['values']['field_project_soft_logo'][0]);
	  }
	  $node->field_sw_project_logo[0] = array('nid' => intval($logo->nid));
	}else{
	  //If there is no logo to save, delete the stored one
	  if(isset($node->field_sw_project_logo[0])&&!empty($node->field_sw_project_logo[0])){
	    node_delete($node->field_sw_project_logo[0]['nid']);
		unset($node->field_sw_project_logo[0]);
	  }
	}
  node_save($node);
  // Set 'project' nid as value for the 'field_project_adms_sw_project' field of the node 'project_project'.
  $form_state['values']['field_project_adms_sw_project']['0']['nid'] = $node->nid;
}

/**
 * Assign a gid to the 'project' node referenced in the 'project_project' node.
 * @param object $node
 *
 * @see isa_node_form_nodeapi
 */
function _isa_node_form_insert_project_project($node) {

  if (isset($node->nid)) {
    if (isset($node->field_project_adms_sw_project['0']['nid'])) {
      $nid = $node->field_project_adms_sw_project['0']['nid'];
      $node_sw = node_load($nid);
      $node_sw->og_groups = array($node->nid => $node->nid);
      $ancestry = array(
        'nid' => $nid,
        'group_nid' => $node->nid,
       );
      drupal_write_record('og_ancestry', $ancestry);
    }
  }
}

/**
 * When a 'project_project' node is deleted, it delete its referenced 'project' node.
 * @param object $node
 *
 * @see isa_node_form_nodeapi
 */
function _isa_node_form_delete_project_project($node) {
  if (isset($node->field_project_adms_sw_project['0']['nid'])) {
    node_delete($node->field_project_adms_sw_project['0']['nid']);
  }
}

/**
 * Redirect to the 'project_project' where the 'project' node is referenced.
 * @param object $node
 *
 * @see isa_node_form_nodeapi
 */
function _isa_node_form_view_project($node) {
  $result = db_query("SELECT nid FROM {content_type_project_project} WHERE field_project_adms_sw_project_nid=%n", $node->nid);
  $fila = db_fetch_array($result);
  if (arg(1) == $node->nid and isset($fila['nid'])) {
    drupal_goto('node/' . $fila['nid']);
  }
}

/**
 * Redirect to the 'project_project' edit form where the 'project' node is referenced.
 * @param object $node
 *
 * @see isa_node_form_nodeapi
 */
function _isa_node_form_edit_project($node) {
  $result = db_query("SELECT nid FROM {content_type_project_project} WHERE field_project_adms_sw_project_nid=%n", $node->nid);
  $fila = db_fetch_array($result);

  if (isset($fila['nid'])) {
    unset($_REQUEST['destination']);
    drupal_goto('node/' . $fila['nid'] . '/edit');
  }
}

function _isa_node_form_create_logo($logo_field,$nid=NULL){
  global $base_url;
  global $user;
  if(isset($logo_field['fid'])&&!empty($logo_field['fid'])){
    if($nid){
	  $logo = node_load($nid);
	}else{
	  $logo = (object) NULL;
	  $logo->type = 'documentation';
	  $logo->language = 'en';
	  $logo->uid = $user->uid;
	  $logo->status = $logo_field['status'];
	}
	$logo->title = $logo_field['filename'];
	$logo->field_documentation_access_url[0]['fid'] = $logo_field['fid'];
	$logo->field_documentation_access_url[0]['list'] = $logo_field['list'];
	$logo->field_documentation_access_url[0]['data'] = $logo_fiel['data'];
	node_save($logo);
	$logo->field_id_uri[0]['value'] = $base_url."/".$logo->path;
	node_save($logo);
  }
  return $logo;
}