<?php

/**
 * Confirm og unsubscription form
 */
function propose_form($form_state, $content_type) {
  global $user;
  $user = user_load($user->uid);

  //Store the content type to be created
  $form['content_type'] = array(
    '#type' => 'value',
    '#value' => $content_type
  );

  //If the content is of type project-project, create the selector for project type
  if ($content_type == 'project-project') {
    $form['description'] = array(
      '#value' => t('Please, select a type of project to create.'),
    );
    $form['project_type'] = array(
      '#type' => 'radios',
      '#title' => t('Project type:'),
      '#options' => array('OSS' => t('Software'), 'asset' => t('Interoperability Solution')),
      '#default_value' => 'OSS',
    );
    $form['destination_cancel'] = array(
      '#type' => 'hidden',
      '#value' => $_GET['destination']
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Continue'),
    );
  }
  elseif ($content_type == 'e-library') {
    $form['description'] = array(
      '#value' => t('Please, select a type of e-Library item to create.'),
    );
    $form['elibrary_type'] = array(
      '#type' => 'radios',
      '#title' => t('e-Library item type:'),
      '#options' => array(
        'presentation' => t('Presentation'),
        'case' => t('Case'),
        'document' => t('Document'),
        'factsheet' => t('Factsheet')
      ),
      '#default_value' => 'presentation',
    );
    $form['destination_cancel'] = array(
      '#type' => 'hidden',
      '#value' => $_GET['destination']
    );
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Continue'),
    );
  }
  else {
    if (!empty($user->og_groups)) {
      //Create the selector for the groups the user is member of
      foreach ($user->og_groups as $gid => $group) {
        if (isa_links_create_content_access($content_type, $gid)) {
          if ($group['title'] == 'Joinup') {
            $group['title'] = t('Joinup (Group)');
          }
          /**
           * For Interoperability solution proposal  don't include communities into the list
           */
          if ($content_type == 'asset-release') {
            if ($group['type'] != 'community') {
              $options[$gid] = $group['title'];
            }
          } else {
            $options[$gid] = $group['title'];
          }
        }
      }
    }

    //Add posibility of adding global news and events
    if ($content_type == 'news' || $content_type == 'event' || $content_type == 'document') {
      $options['site'] = t('Joinup (Site)');
    }


    //If the user has the right permissions, allow him to create content inside the group that is being viewed
    if (isa_links_create_content_access($content_type, arg(2)) && arg(2)) {
      $options[arg(2)] = 'Current group';
    }

    //Set the group that is currently being displayed as the default value.
    $default_value = NULL;
    if (arg(2) && isset($options[arg(2)])) {
      $default_value = arg(2);
    }
    
    //If the user can propose content in any group, create the form
    if (!empty($options)) {
      $form['description'] = array(
        '#value' => t('Please, select an appropriate group to add your content to.'),
      );
      
      //set default value to first optionskey if none
      if (!$default_value) {
        $default_value = key($options);
      }
      
      $form['groups'] = array(
        '#type' => 'radios',
        '#title' => t('Groups you belong to:'),
        '#options' => $options,
        '#default_value' => $default_value,
      );
      $form['destination_cancel'] = array(
        '#type' => 'hidden',
        '#value' => $_GET['destination']
      );
      $form['submit'] = array(
        '#type' => 'submit',
        '#value' => t('Continue'),
      );
    }
    else {
      //If the user doesn't belong to any group, don't create any form and show a message instead.
      $form['description'] = array(
        '#value' => t('You do not belong to any community inside Joinup. Please, apply for a community before trying to propose any content.'),
      );
    }
  }

  return $form;
}

/**
 * Confirm og unsubscription submit handler
 */
function propose_form_submit($form, &$form_state) {
  $path = 'node/add/' . $form_state['values']['content_type'];
  if (isset($form_state['values']['groups']) && !empty($form_state['values']['groups'])) {
    //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1246
    //Redirection when cancelling
    if ($form_state['values']['groups'] == 'site') {
      $query = 'destination_cancel=' . $form_state['values']['destination_cancel'];
    }
    else {
      $query = 'gids[]=' . $form_state['values']['groups'] . '&destination_cancel=' . $form_state['values']['destination_cancel'];
    }
  }
  elseif (!empty($form_state['values']['project_type'])) {
    $query = 'type=' . $form_state['values']['project_type'] . '&destination_cancel=' . $form_state['values']['destination_cancel'];
  }
  elseif (!empty($form_state['values']['elibrary_type'])) {
    $path = 'node/add/' . $form_state['values']['elibrary_type'];
    $query = 'destination_cancel=' . $form_state['values']['destination_cancel'];
  }
  if (isset($_SERVER["HTTP_X_REQUESTED_WITH"])) {
    $_SESSION["form_redirect"] = $path;
    $_SESSION["form_redirect_query"] = $query;
    if (function_exists("popups_render_as_json") && isset($_REQUEST["destination"])) {
      unset($_SESSION['page_override']);
      return drupal_json(array(
        'title' => drupal_get_title(),
        'messages' => theme('status_messages'),
        'path' => $_REQUEST["destination"],
        'content' => "",
      ));
    }
  }
}
