<?php

/**
 * Query the database to count the number of not blocked users
 *
 * @return int
 */
function isa_core_feature_count_users() {
  $query = "SELECT COUNT(uid) FROM {users} WHERE status = %d AND login != 0";
  return (int) db_result(db_query($query, ISA_CORE_ACTIVE_USER_STATUS));
}

/**
 * Query the database to count the number of published communities
 *
 * @return int
 */
function isa_core_feature_count_communities() {
  $query = "SELECT COUNT(n.nid)
    FROM {node} n
    INNER JOIN {workflow_node} wn ON wn.nid = n.nid
    WHERE n.type = '%s' AND n.status = 1 AND wn.sid = '%s'";
  return (int) db_result(db_query($query, ISA_CORE_COMMUNITY_CONTENT_TYPE_NAME, ISA_CORE_COMMUNITY_VALIDATED_STATE));
}

/**
 * Query the database to count the number of published interoperability solutions
 * @param int $solution_type_id 
 *  Term ID of the solution type as optional
 * @return int
 */
function isa_core_feature_count_interoperability_solutions($solution_type_id = NULL) {
  
  $query = "SELECT COUNT(DISTINCT n.nid)
    FROM {node} n
    INNER JOIN {workflow_node} wn ON wn.nid = n.nid ";
  
  //joinu term_node table to put add condition on solution type term id
  if (!empty($solution_type_id)) {
    $query .= " INNER JOIN {term_node} tn ON tn.nid = n.nid ";
  }
  
  $query .= "WHERE n.type = '%s' AND n.status = 1 AND wn.sid = '%s'";
  
  //if solution type 
  if (!empty($solution_type_id)) {
    $query .= " AND tn.tid = %d";
    $count = (int) db_result(db_query($query, ISA_CORE_INTEROPERABILITY_SOLUTION_CONTENT_TYPE_NAME, ISA_CORE_INTEROPERABILITY_SOLUTION_VALIDATED_STATE, $solution_type_id));
  } else {
    $count = (int) db_result(db_query($query, ISA_CORE_INTEROPERABILITY_SOLUTION_CONTENT_TYPE_NAME, ISA_CORE_INTEROPERABILITY_SOLUTION_VALIDATED_STATE));
  }
  
  return $count;
}

function isa_core_federated_repositories() {
  $query = "SELECT COUNT(nid)
    FROM {node}
    WHERE type = '%s' AND status = 1";
  return (int) db_result(db_query($query, ISA_CORE_FEDERATED_REPOSITORY_CONTENT_TYPE));
}