<?php
// $Id: node.tpl.php,v 1.4.2.1 2009/08/10 10:48:33 goba Exp $

/**
 * @defgroup nodes
 * @ingroup templates
 *
 * @file
 *
 * Default theme implementation for node.
 *
 * Available variables:
 * - $title: the (sanitized) title of the node.
 * - $content: Node body or teaser depending on $teaser flag.
 * - $picture: The authors picture of the node output from
 *   theme_user_picture().
 * - $date: Formatted creation date (use $created to reformat with
 *   format_date()).
 * - $links: Themed links like "Read more", "Add new comment", etc. output
 *   from theme_links().
 * - $name: Themed username of node author output from theme_username().
 * - $node_url: Direct url of the current node.
 * - $terms: the themed list of taxonomy term links output from theme_links().
 * - $submitted: themed submission information output from
 *   theme_node_submitted().
 *
 * Other variables:
 * - $node: Full node object. Contains data that may not be safe.
 * - $type: Node type, i.e. story, page, blog, etc.
 * - $comment_count: Number of comments attached to the node.
 * - $uid: User ID of the node author.
 * - $created: Time the node was published formatted in Unix timestamp.
 * - $zebra: Outputs either "even" or "odd". Useful for zebra striping in
 *   teaser listings.
 * - $id: Position of the node. Increments each time it's output.
 *
 * Node status variables:
 * - $teaser: Flag for the teaser state.
 * - $page: Flag for the full page state.
 * - $promote: Flag for front page promotion state.
 * - $sticky: Flags for sticky post setting.
 * - $status: Flag for published status.
 * - $comment: State of comment settings for the node.
 * - $readmore: Flags true if the teaser content of the node cannot hold the
 *   main body content.
 * - $is_front: Flags true when presented in the front page.
 * - $logged_in: Flags true when the current user is a logged-in member.
 * - $is_admin: Flags true when the current user is an administrator.
 *
 * @see joinup_preprocess_node()
 * @see ()
 * @ingroup nodes
 */
global $base_url;

?>
<div id="node-<?php print $node->nid; ?><?php if (!$page&&!$teaser) { print '-blockinfo'; } ?>" class="node<?php  print (!$status ? ' node-unpublished':'');?> node-type-<?php print $node->type; ?> clear-block">
	<div class="node-content" id="content-border">
	  <?php if ($page): ?>
		<div class="field field-submitted"><?php print $submitted; ?></div>
		<div class="field field-repository-content field-content-body">
			<h3 class="page-subtitle-content"><?php print t('Description'); ?></h3>
			<div class='field-item'><?php print $description; ?></div>
		</div>

		<br class="clear" />
		<div class="terms_keywords">
		<?php if (!empty($taxonomy_terms_keywords['value'])): ?>
		  <h3  class="keywords-subtitle "><?php print $taxonomy_terms_keywords['label']; ?></h3>
		  <?php print $taxonomy_terms_keywords['value']; ?>
	    <?php endif; ?>
		</div>
		<div class="terms_themes">
	    <?php if (!empty($taxonomy_terms_themes['value'])): ?>
		  <h3  class="keywords-subtitle "><?php print $taxonomy_terms_themes['label']; ?></h3>
		  <?php print $taxonomy_terms_themes['value']; ?>
	    <?php endif; ?>
		</div>
		<?php
			// https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-909
			// User request - Federated repository page
			// The list of asset releases should be by default "unfolded".
		?>
		<div class="toggable-asset-repository">
			<h4 class="togglable-expanded"><?php print t('Interoperability Solutions') .
			'<img class="collapsed" src="' . $base_url . '/' . path_to_theme(). '/images/forms/fieldset/menu-collapsed-blue.png"
			  alt="Additional information" title="Additional information"/>
			<img class="expanded" src="' . $base_url . '/' . path_to_theme(). '/images/forms/fieldset/menu-expanded-blue.png"
			  alt="Hide more information" title="Hide more information"/>' ; ?></h4>
			<div class="toggle-wrapper" style="display:block;">
				<?php
					// https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-909
					// User request - Federated repository page
					// 2. We should also add a button with direct link to the facetted search from the repository page
					$params = array(
						'query' => array(
									'retain-filters' 		=> 1,
									'hidden_type' 			=> 1,
									'filters'				=> 'sm_facetbuilder_facet_node_type:"facet_node_type:facet_24" is_repository_origin:'.$node->nid . ' ss_release_type:"asset"',
								),
						'attributes' => array('id' => 'advanced_search_link')
					);
					$default = '';
				?>

				<div id="advanced-search-repository" class="form-item">
					<?php print l(t('Explore this repository'),'search/apachesolr_search/' . $default, $params);?>
				</div>
        
        <?php $interoperability_values = isa_federated_get_interoperability_solutions($node); ?>
        <?php if (!empty($interoperability_values)) : ?>
        <div class="interoperability-solutions-list view content-colspans-8 view-asset-release">
          <div class="view-content">
          <?php foreach ($interoperability_values as $r) : ?>
            <?php print '<div class="odd views-row-first clearfix">'; ?>
                <?php print '<div class="colspan-0  first last fields views-field-title">'; ?>
                  <?php print '<div class="field field-title">' . $r['title']. '</div>'; ?>
                  <?php print '<div class="field field-revisions-teaser">' . $r['teaser'] . '</div>'; ?>
                <?php print '</div>'; ?>
              <?php print '</div>'; ?>
          <?php endforeach; ?>
          
          <?php print theme('pager', NULL, 10, 0); ?>
          </div>
        </div>
        <?php else: ?>
        <div class="view-empty">
          <p>There is no validated asset release in this repository<br></p>
        </div>
        <?php endif; ?>
        
			</div>
		</div>

	  <?php else: ?>
		<div id="node-information" class="box information">
			<div class="odd nodes-row-first nodes-row-last clearfix">

              <?php if (!empty($field_id_uri['0']['value'])||!empty($field_repository_url['0']['value'])): ?>
				<div class="box-links-select">
                  <select class="links-select">
                    <?php if (!empty($field_repository_url['0']['url'])): ?>
                      <option class="option field-homepage-term" value="<?php print $field_repository_url['0']['url']; ?>"><?php print t('Access URL'); ?></option>
                    <?php endif; ?>
                    <?php if (!empty($field_id_uri['0']['value'])): ?>
                      <option class="option field-homepage-term" value="<?php print $field_id_uri['0']['value']; ?>"><?php print t('ID (URI)'); ?></option>
                    <?php endif; ?>
                  </select>
                  <input class='links-value' type="text" readonly="readonly"></input>
				</div>
              <?php endif; ?>
                
                <h3 class="com-side-title"><?php print t('Metadata'); ?></h3>
                
                <div class="com-property">
                    <span class="com-property-title"><?php print t('Type'); ?>:</span>
                    <span class="com-property-value"><?php print node_get_types('name', $node->type); ?></span>
                </div>

                <?php if (!empty($owner['uid'])): ?>
                <div class="com-property">
                    <div class="com-abstract-title"><?php print t('Owner') ?>:</div>
                    <div class="com-abstract-description"><a href="/people/<?php print $owner['uid']; ?>"><?php print $owner['name']; ?></a></div>
                </div>
                <?php endif; ?>
                
                <?php if (!empty($field_repository_harvestmethod['value'])): ?>
                <div class="com-property">
                    <span class="com-property-title"><?php print t('Metadata upload module') ?>:</span>
                    <span class="com-property-value"><?php print $field_repository_harvestmethod['value']; ?></span>
                </div>
                <?php endif; ?>
        
                <?php if (!empty($report_file['value'])): ?>
                <div class="com-property">
                    <div class="com-abstract-title"><?php print t('Report file') ?>:</div>
                    <div class="com-abstract-description"><?php print $report_file['value']; ?></div>
                </div>
                <?php endif; ?>

                <?php if (!empty($field_repository_publisher['value'])): ?>
                <div class="com-property">
                    <div class="com-abstract-title"><?php print t('Publisher') ?>:</div>
                    <div class="com-abstract-description"><?php print $field_repository_publisher['value']; ?></div>
                    <div id="field_repository_publisher"></div>
                </div>
                <?php endif; ?>
    
                <?php if (!empty($field_repository_contact_point['value'])): ?>
                <div class="com-property">
                    <span class="com-property-title"><?php print $field_repository_contact_point['label'] ?>:</span>
                    <span class="com-property-value"><?php print $field_repository_contact_point['value']; ?></span>
                    <div id="field_repository_contact_point"></div>
                </div>
                <?php endif; ?>
        
                <?php if (!empty($field_repository_asset_reference['value'])): ?>
                <div class="com-property">
                    <span class="com-property-title"><?php print t('Referenced assets') ?>:</span>
                    <span class="com-property-value"><?php print $field_repository_asset_reference['value']; ?></span>
                    <div id="field_repository_asset_reference"></div>
                </div>
                <?php endif; ?>
        
                <?php if (!empty($field_repository_schema['value'])): ?>
                <div class="com-property">
                    <span class="com-property-title"><?php print t('Schema') ?>:</span>>
                    <span class="com-property-value"><?php print $field_repository_schema['value']; ?></span>
                </div>
                <?php endif; ?>
                    
                <?php if (!empty($taxonomy_terms_geographiccoverage['value'])): ?>
                <div class="com-property">
                    <span class="com-property-title"><?php print t('Geographic coverage') ?>:</span>
                    <span class="com-property-value"><?php print $taxonomy_terms_geographiccoverage['value']; ?></span>
                </div>
                <?php endif; ?>
        
                <?php if (!empty($date_created['value'])): ?>
                <div class="com-property">
                    <span class="com-property-title"><?php print t('Date created') ?>:</span>
                    <span class="com-property-value"><?php print $date_created['value']; ?></span>
                </div>
                <?php endif; ?>
                
                <?php if (!empty($field_asset_last_modification['value'])): ?>
                <div class="com-property">
                    <span class="com-property-title"><?php print t('Date last modification') ?>:</span>
                    <span class="com-property-value"><?php print $field_asset_last_modification['value']; ?></span>
                </div>
                <?php endif; ?>
                
                <div class="com-property last">
                  <span class="com-property-title"><?php echo t('Validate') ?>: </span>
                  <span class="com-property-value"><?php echo isa_adms_validation_link($field_repository_harvestmethod['value']); ?></span>
                </div>	
			</div>
            
			<div class="node-metadata-buttons">
				<a class="links export-metadata" target="_blank" title="Export a description metadata of your repository" href="<?php print base_path()?>catalogue/repository/<?php print $node->nid; ?>/rdf">
					<span>Export metadata</span>
				</a>
				<?php if ($node->field_repository_harvestmethod[0]['value'] != 1): ?>
					<a class="links upload-metadata" target="_blank" title="Upload a description metadata in your repository" href="<?php echo base_path() ?>repository/upload-metadata/<?php print $node->nid;?>?destination=node/<?php print $node->nid;?>">
							<span>Upload metadata</span>
					</a>
				<?php endif; ?>
			</div>
		</div>
	  <?php endif; ?>
	</div>
</div>