<?php
/**
 * ADMSxImport/export testing script
 * This script uses the AdmsTestFramework to automate the import and export process and generate files to validate the process
 * It works as follow:
 *  .- Launch the script with the original rdf file to import 
 *  [the rest is automated]
 *  .- Copy the rdf file to local public url, to use it as harvest url [original_import.rdf]
 *  .- Parse the file to create the repository skeleton
 *  .- Launch the src_adms/cronjob to import from original_import.rdf
 *  .- Save the repository tree structure, with all its related nodes [original_import.json]
 *  .- Export the repository to rdf [original_import_export.rdf]
 *  .- Delete all the nodes from the repository structure
 *  .- Repeat the previous process using the exported rdf [original_import_export.rdf] 
 *  .- Two new files will be created
 *      .- [import_import.json] with the repository structure
 *      .- [imported_export.rdf] with the rdf export from the repository structure
 *  
 *  Now we can compare the files to verify everything is correct:
 *  .- Valid ADMS specification for files:
 *     .- original_import.rdf [ADMS(v1.02)|ADMS.sw|ADMSx]
 *     .- original_import_export.rdf [ADMSx]
 *     .- imported_export.rdf [ADMSx]
 *  .- [original_import.rdf | original_import_export.rdf | imported_export.rdf] should be validated using http://www.w3.org/RDF/Validator/
 *  .- [original_import.rdf | original_import_export.rdf | imported_export.rdf] should contain the same objects, but can use different specifications 
 *  .- [original_import.json and import_import.json] should be equal, except for dates, nids and vids
 */
ini_set('display_errors', 1);
ini_set('log_errors', 1);
ini_set('error_log', dirname(__FILE__) . '/error_log.txt');
error_reporting(E_ALL & ~E_NOTICE);
$_SERVER['SCRIPT_NAME'] = "/";

require_once 'bootstrap.php';
require_once( dirname(__FILE__) . '/test/AdmsTestFramework.php');
set_include_path(get_include_path() . PATH_SEPARATOR . '../lib/easyrdf/lib/');
require_once "test/easyrdf/lib/EasyRdf.php";

try{
  $admsTest =  new \test\AdmsTestFramework\AdmsTestFramework;
  $admsTest->configure($argv); 
  isset($argv[1]) ? $_SERVER['HTTP_HOST'] = $argv[1] : FALSE;
  include_once './includes/bootstrap.inc';
  drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);  
  $admsTest->launch(); 
} catch (Exception $e)  {
  print($e);
}

