<?php

namespace ADMSx\RdfClasses;

use Common\ADMSObject;
use Common\iADMSObject;

/**
 * class Asset
 *
 */
class Asset extends ADMSObject implements iADMSObject {

  /**
   * Contains the defined types of the asset that could exist
   * @access protected
   */
  public $type = array(
      'admsx' => 'http://www.w3.org/ns/adms#Asset',
      'admssw' => 'http://www.w3.org/ns/radion#Asset',
      'admssw_2' => 'http://purl.org/adms/sw/SoftwareRelease',
      'adms' => 'http://www.w3.org/ns/adms#SemanticAsset'
  );
  public $content_type = 'asset_release';

  /**
   * Contains the properties of the Asset
   * @access protected
   */
  public $properties = array(
      'alternativeName' => array(
          'namespace' => array(
              'admsx' => 'dct:alternative',
          ),
          'cardinality' => array(0, 0),
          'map' => 'literal',
          'node_field' => '',
          'value_key' => '',
          'extra_language' => array('field_asset_alternative_name'),
          'extra_language_type' => 'language_textfield'
      ),
      'created' => array(
          'namespace' => array(
              'admsx' => 'dct:issued',
              'adms' => 'dcterms:created',
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'literal',
          'node_field' => 'created',
          'value_key' => 'created'
      ),
      'theme' => array(
          'namespace' => array(
              'admsx' => 'dcat:theme',
          ),
          'cardinality' => array(1, 0),
          'map' => 'property',
          'taxonomy_field' => ISA_VID_THEMES
      ),
      'description' => array(
          'namespace' => array(
              'admsx' => 'dct:description',
              'admssw' => 'doap:description',
          ),
          'map' => 'literal',
          'node_field' => 'body',
          'cardinality' => array(1, 0),
          'type' => '',
          'extra_language' => array('field_asset_description'),
          'extra_language_type' => 'language_textarea'
      ),
      'modified' => array(
          'namespace' => array(
              'admsx' => 'dct:modified',
              'adms' => 'dcterms:modified',
          ),
          'cardinality' => array(1, 1),
          'type' => '',
          'map' => 'literal',
          'node_field' => 'changed',
          'value_key' => 'modified'
      ),
      'publisher' => array(
          'namespace' => array(
              'admsx' => 'dct:publisher',
              'adms' => 'dcterms:publisher'
          ),
          'cardinality' => array(1, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_publisher',
          'value_key' => 'nid'
      ),
      'title' => array(
          'namespace' => array(
              'admsx' => 'dct:title',
              'adms' => 'rdfs:label'
          ),
          'cardinality' => array(1, 0),
          'type' => '',
          'map' => 'literal',
          'node_field' => 'title',
          'value_key' => '',
          'extra_language' => array('field_asset_name'),
          'extra_language_type' => 'language_textfield'
      ),
      'solutionType' => array(
          'namespace' => array(
              'admsx' => 'dct:type',
          ),
          'map' => 'literal',
          'type' => '',
          'taxonomy_field' => ISA_VID_TYPE,
          'cardinality' => array(1, 0)
      ),
      'status' => array(
          'namespace' => array(
              'admsx' => 'adms:status',
              'adms' => 'trove:status',
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'property',
          'taxonomy_field' => ISA_VID_STATUS,
      ),
      'interoperabilityLevel' => array(
          'namespace' => array(
              'admsx' => 'adms:interoperabilityLevel'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'taxonomy_field' => ISA_VID_INTEROPERABILITY_LEVEL,
      ),
      'contactPoint' => array(
          'namespace' => array(
              'admsx' => 'dcat:contactPoint',
              'adms' => 'adms:contactPoint',
              'admssw' => 'dcterms:contactPoint',
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_contact_point',
          'value_key' => 'nid'
      ),
      'distribution' => array(
          'namespace' => array(
              'admsx' => 'dcat:distribution',
              'adms' => 'rad:distribution',
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_distribution',
          'value_key' => 'nid'
      ),
      'keyword' => array(
          'namespace' => array(
              'admsx' => 'dcat:keyword',
              'admssw' => 'rad:keyword'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'literal',
          'taxonomy_field' => ISA_VID_KEYWORDS,
          'taxonomy_literal' => true,
          'taxonomy_create' => true
      ),
      'landingPage' => array(
          'namespace' => array(
              'admsx' => 'dcat:landingPage',
              'adms' => 'foaf:homepage',
              'admssw' => 'doap:homepage'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_homepage_doc',
          'value_key' => 'nid'
      ),
      'language' => array(
          'namespace' => array(
              'admsx' => 'dct:language',
              'adms' => 'dcterms:language',
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_language_multiple',
          'value_key' => 'taxonomy'
      ),
      'relation' => array(
          'namespace' => array(
              'admsx' => 'dct:relation'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_node_reference',
          'value_key' => 'nid'
      ),
      'spatial' => array(
          'namespace' => array(
              'admsx' => 'dct:spatial'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'taxonomy_field' => ISA_VID_GEOGRAPHIC_COVERAGE,
      ),
      'temporal' => array(
          'namespace' => array(
              'admsx' => 'dct:temporal'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'periodOfTime',
          'value_key' => 'nid'
      ),
      'logo' => array(
          'namespace' => array(
              'admsx' => 'foaf:logo'
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_sw_logo',
          'value_key' => 'nid'
      ),
      'identifier' => array(
          'namespace' => array(
              'admsx' => 'adms:identifier',
              'admssw' => 'admssw:identifier'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_identifier',
          'value_key' => 'item_id'
      ),
      'includedAsset' => array(
          'namespace' => array(
              'admsx' => 'adms:includedAsset'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_node_reference',
          'value_key' => 'nid',
          'extra_switch' => array('field_asset_node_relationship' => 4),
          'extra_switch_type' => 'asset_node_reference'
      ),
      'last' => array(
          'namespace' => array(
              'admsx' => 'adms:last'
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_current_reference',
          'value_key' => 'nid',
          'extra_switch' => array('field_asset_current_reference' => "no"),
          'extra_switch_type' => 'asset_node_reference'
      ),
      'next' => array(
          'namespace' => array(
              'adms' => 'xhv:next',
              'admsx' => 'adms:next',
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_node_reference',
          'value_key' => 'nid',
          'extra_switch' => array('field_asset_node_relationship' => 0),
          'extra_switch_type' => 'asset_node_reference'
      ),
      'prev' => array(
          'namespace' => array(
              'adms' => 'xhv:prev',
              'admsx' => 'adms:prev'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_node_reference',
          'value_key' => 'nid',
          'extra_switch' => array('field_asset_node_relationship' => 2),
          'extra_switch_type' => 'asset_node_reference'
      ),
      'sample' => array(
          'namespace' => array(
              'admsx' => 'adms:sample'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_node_reference',
          'value_key' => 'nid',
          'extra_switch' => array('field_asset_node_relationship' => 6),
          'extra_switch_type' => 'asset_node_reference'
      ),
      'translation' => array(
          'namespace' => array(
              'admsx' => 'adms:translation'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_node_reference',
          'value_key' => 'nid',
          'extra_switch' => array('field_asset_node_relationship' => 3),
          'extra_switch_type' => 'asset_node_reference'
      ),
      'versionNotes' => array(
          'namespace' => array(
              'admsx' => 'adms:versionNotes',
              'admssw' => 'rad:versionNotes'
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'literal',
          'node_field' => '',
          'value_key' => '',
          'extra_language' => array('field_asset_version_note'),
          'extra_language_type' => 'language_textarea'
      ),
      'version' => array(
          'namespace' => array(
              'admsx' => 'owl:versionInfo',
              'admssw' => 'rad:version'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'literal',
          'node_field' => '',
          'value_key' => '',
          'extra_language' => array('field_asset_version'),
          'extra_language_type' => 'language_textfield'
      ),
      'issued' => array(
          'namespace' => array(
              'admsx' => 'dct:issued'
          ),
          'cardinality' => array(0, 1),
          'type' => '',
          'map' => 'literal',
          'node_field' => 'field_asset_metadata_date',
          'value_key' => 'date'
      ),
      'page' => array(
          'namespace' => array(
              'admsx' => 'adms:relatedWebPage',
              'adms' => 'foaf:page'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_webpage_doc',
          'value_key' => 'nid'
      ),
      'versionInfo' => array(
          'namespace' => array(
              'admsx' => 'owl:versionInfo'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_version',
          'value_key' => 'nid'
      ),
      'altLabel' => array(
          'namespace' => array(
              'admsx' => 'skos:altLabel'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'literal',
          'node_field' => 'field_asset_alternative_name',
          'value_key' => 'nid'
      ),
      'describedBy' => array(
          'namespace' => array(
              'admsx' => 'wdrs:describedBy'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_documentation',
          'value_key' => 'nid'
      ),
      'metrics' => array(
          'namespace' => array(
              'admsx' => 'admssw:metrics'
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_sw_metrics',
          'value_key' => 'nid'
      ),
      'isPartOf' => array(
          'namespace' => array(
              'adms' => 'dcterms:isPartOf',
          ),
          'cardinality' => array(0, 1),
          'map' => 'property',
          'organic_group_field' => true
      ),
      'assessment' => array(
          'namespace' => array(
              'admsx' => 'foaf:document',
              'adms' => 'admssw:assessment',
          ),
          'cardinality' => array(0, 0),
          'type' => '',
          'map' => 'property',
          'node_field' => 'field_asset_assessment',
          'value_key' => 'nid'
      ),
  );

  public function __construct() {
    parent::__construct();
  }

  protected function importEntitySpecificParameters() {
    //for asset_release/asset matching
    //change type and assign og_group
    //allow commenting asset_release
    $node = &$this->node;
    $node->og_groups[$this->output['repository_id']] = $this->output['repository_id'];
    $node->comment = COMMENT_NODE_READ_WRITE;
    //To remove when implementind ADMS SW import/export: save release as asset
    $node->field_asset_release_type[0]['value'] = "asset";
    $this->node = $node;
  }

}
