<?php

namespace ADMSx\RdfClasses;

/**
 * class AssetDistribution
 *
 */
class AssetDistribution extends \Common\ADMSObject implements \Common\iADMSObject {

  /**
   * Contains the defined types of the asset distribution that could exist
   * @access protected
   */
  public $type = array(
    'admsx' => 'http://www.w3.org/ns/adms#AssetDistribution',
    'adms' => 'http://www.w3.org/ns/adms#SemanticAssetDistribution',
    'admssw' => 'http://purl.org/adms/sw/SoftwarePackage',
    );

  public $content_type = 'distribution';

  /**
   * Contains the properties of the Asset Distribution
   * @access protected
   */
  public $properties = array(
    'accessURL' => array(
      'namespace' => array(
         'admsx' => 'dcat:accessURL',
         'adms' => 'adms:accessURL'
        ),
      'cardinality' => array(1, 0),
      'map' => 'property',
      'node_field' => 'field_distribution_access_url1',
      'value_key' => 'url'
      ),
    'status' => array(
      'namespace' => array(
        'admsx' => 'adms:status',
        ),
      'cardinality' => array(0, 1),
      'map' => 'property',
      'taxonomy_field' => ISA_VID_STATUS,
      ),
    'downloadURL' => array(
      'namespace' => array(
        'admsx' => 'dcat:downloadURL',
        ),
      'cardinality' => array(0, 0),
      'map' => 'property',
      'node_field' => 'field_distribution_download_url',
      'value_key' => 'url'
      ),
    'mediaType' => array(
      'namespace' => array(
        'admsx' => 'dcat:mediaType',
        'adms' => 'dcterms:format'
        ),
      'cardinality' => array(0, 1),
      'map' => 'property',
      'taxonomy_field' => ISA_VID_FILE_FORMAT,
      ),
    'licence' => array(
      'namespace' => array(
        'admsx' => 'dct:licence',
        'adms' => 'dcterms:license',
        ),
      'cardinality' => array(0, 1),
      'map' => 'property',
      'node_field' => 'field_distribution_licence',
      'value_key' => 'nid'
      ),
    'representationTecnique' => array(
      'namespace' => array(
        'admsx' => 'adms:representationTechnique',
        'adms' => 'skos:Concept',
        ),
      'cardinality' => array(0, 1),
      'map' => 'property',
      'taxonomy_field' => ISA_VID_REPRESENTATION_TECNIQUE,
      ),
    'description' => array(
      'namespace' => array(
        'admsx' => 'dct:description',
        'admssw' => 'dcterms:description ',
        ),
      'cardinality' => array(0,0),
      'type' => '',
      'map' => 'literal',
      'node_field' => 'body',
      'value_key' => '',
      'extra_language' => array('field_distribution_description'),
      'extra_language_type' => 'language_textarea'
      ),
    'modified' => array(
      'namespace' => array(
        'admsx' => 'dct:modified',
        'admssw' => 'dcterms:modified',
      ),
      'cardinality' => array(0, 1),
      'type' => '',
      'map' => 'literal',
      'node_field' => 'field_asset_last_modification',
      'value_key' => 'date'
    ),
    'publisher' => array(
      'namespace' => array(
        'admsx' => 'dct:publisher'
        ),
      'map' => 'property',
      'node_field' => 'field_distribution_publisher',
      'value_key' => 'nid',
      'cardinality' => array(1,0),
      'type' => '',
      ),
    'name' => array(
      'namespace' => array(
        'admsx' => 'dct:title',
        'adms' => 'rdfs:label'
        ),
      'cardinality' => array(1,0),
      'type' => '',
      'map' => 'literal',
      'node_field' => 'title',
      'value_key' => '',
      'extra_language' => array('field_distribution_name'),
      'extra_language_type' => 'language_textfield'
      ),
    'filesize' => array(
      'namespace' => array(
        'admsx' => 'schema:fileSize',
        ),
      'cardinality' => array(0,1),
      'type' => '',
      'map' => 'property',
      'node_field' => 'field_distribution_sw_filesize',
      'value_key' => 'value',
      ),
    'checksum' => array(
      'namespace' => array(
        'admsx' => 'spdx:checksum',
      ),
      'cardinality' => array(0, 1),
      'map' => 'property',
      'node_field' => 'field_distribution_sw_checksum',
      'value_key' => 'nid'
    ),
    'software_id' => array(
      'namespace' => array(
        'admsx' => 'swid:software_id',
      ),
      'cardinality' => array(0, 1),
      'map' => 'property',
      'node_field' => 'field_distribution_sw_identifier',
      'value_key' => 'nid'
    ),
    'created' => array(
      'namespace' => array(
        'admsx' => 'dct:issued',
        'adms' => 'dcterms:created',
        ),
      'cardinality' => array(0,1),
      'type' => '',
      'map' => 'literal',
      'node_field' => 'created',
      'value_key' => 'created'
      ),
    'tagUrl' => array(
      'namespace' => array(
        'admsx' => 'admssw:tagURL',
        ),
      'cardinality' => array(0,1),
      'type' => '',
      'map' => 'property',
      'node_field' => 'field_distribution_sw_tagurl',
      'value_key' => 'url'
      ),
    );

public function __construct() {
  parent::__construct();
}
}
