<?php

namespace ADMSx\RdfClasses;

/**
 * class AssetRepository
 *
 */
class AssetRepository extends \Common\ADMSObject implements \Common\iADMSObject {

  /**
   * Contains the defined types of the asset repository that could exist
   * @access protected
   */
  public $type = array(
      'admsx' => 'http://www.w3.org/ns/adms#AssetRepository',
      'adms' => 'http://www.w3.org/ns/adms#SemanticAssetRepository',
      'admssw' => 'http://purl.org/adms/sw/SoftwareRepository',
    );

  public $content_type = 'repository';

  /**
   * Contains the properties of the Asset Repository
   * @access protected
   */
  public $properties = array(
      'accessURL' => array(
        'namespace' => array(
          'admsx' => 'dcat:accessURL',
          'adms' => 'adms:accessURL',
          'admssw' => 'admssw:accessURL'
        ),
        'cardinality' => array(1,0),
        'map' => 'property',
        'node_field' => 'field_repository_url',
        'value_key' => 'url'
      ),
      'name' => array(
        'namespace' => array(
          'admsx' => 'dct:title',
          'adms' => 'rdfs:label',
        ),
        'cardinality' => array(1,0),
        'type' => '',
        'map' => 'literal',
        'node_field' => 'title',
        'value_key' => '',
        'extra_language' => array('field_repository_name'),
        'extra_language_type' => 'language_textfield'
      ),
      'contactPoint' => array(
        'namespace' => array(
          'admsx' => 'dcat:contactPoint',
          'adms' => 'adms:contactPoint',
          'admssw' => 'dcterms:contactPoint',
        ),
        'cardinality' => array(0,0),
        'type' => '',
        'map' => 'property',
        'node_field' => 'field_repository_contact_point',
        'value_key' => 'nid'
      ),
      'description' => array(
        'namespace' => array(
          'admsx' => 'dct:description',
          'adms' => 'dcterms:description',
        ),
        'map' => 'literal',
        'node_field' => 'body',
        'cardinality' => array(1,0),
        'type' => '',
        'value_key' => '',
        'extra_language' => array('field_repository_description'),
        'extra_language_type' => 'language_textarea'
      ),
      'publisher' => array(
        'namespace' => array(
          'admsx' => 'dct:publisher',
          'adms' => 'dcterms:publisher'
        ),
        'cardinality' => array(1,0),
        'type' => '',
        'map' => 'property',
        'node_field' => 'field_repository_publisher',
        'value_key' => 'nid'
      ),
      'themeTaxonomy' => array(
        'namespace' => array(
          'admsx' => 'dcat:themeTaxonomy',
          'adms' => 'dcterms:subject',
          'admssw' => 'rad:themeTaxonomy',
        ),
        'cardinality' => array(0,0),
        'taxonomy_field' => ISA_VID_THEMES,
        'map' => 'property',
        'type' => ''
      ),
      'spatial' => array(
        'namespace' => array(
          'admsx' => 'dct:spatial',
          'adms' => 'dcterms:spatial',
        ),
        'cardinality' => array(0,0),
        'taxonomy_field' => ISA_VID_GEOGRAPHIC_COVERAGE,
        'type' => '',
        'map' => 'property',
      ),
      'supportedSchema' => array(
        'namespace' => array(
          'admsx' => 'adms:supportedSchema'
        ),
        'cardinality' => array(0,0),
        'type' => '',
        'map' => 'literal',
        'node_field' => '',
        'value_key' => '',
        'extra_language' => array('field_repository_schema'),
        'extra_language_type' => 'language_textfield'
      ),
      'created' => array(
        'namespace' => array(
          'admsx' => 'dct:issued',
          'adms' => 'dcterms:created',
        ),
        'cardinality' => array(0,1),
        'type' => '',
        'map' => 'literal',
        'node_field' => 'created',
        'value_key' => 'created'
      ),
      'modified' => array(
        'namespace' => array(
          'admsx' => 'dct:modified',
          'adms' => 'dcterms:modified',
        ),
        'cardinality' => array(1,1),
        'type' => '',
        'map' => 'literal',
        'node_field' => 'changed',
        'value_key' => 'modified'
      ),
      'haspart' => array(
        'namespace' => array(
          'admsx' => 'dcterms:hasPart',
        ),
        'cardinality' => array(0,0),
        'type' => '',
        'map' => 'property',
        'node_field' => 'field_repository_asset_reference',
        'value_key' => 'nid'
      ),
     'dataset' => array(
      'namespace' => array(
          'admsx' => 'dcat:dataset',
          'adms' => 'adms:Asset',
        ),
      'map' => 'property',
      'organic_group_field' => TRUE
      ),
  );

  public function __construct() {
    parent::__construct();
  }
}
