<?php

/*
 * Copyright (C) Atos
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

namespace Adapters;

/**
 * This class is implementing the design pattern "Factory" in order to manage
 * the creation of Loaders.
 */
class AdapterFactory {

  /**
   * This function is retrieving the correct loader based on the parameter
   * @param string type the type of loader needed
   * @return ILoader an object of Loader type
   * @access public
   * @throws UnexpectedValueException if the type is not registered
   */
  //sites/default/files/trash/default.xml
  public static function getAdapter($mimetype = 'application/rdf+xml', $filepath = '', $debugLevel = '') {
    if ('' != $filepath) {
      // Creating the log instance
      \Logger::configure(array(
        'rootLogger' => array(
            'appenders' => array('default'),
        ),
        'appenders' => array(
            'default' => array(
              'class' => 'LoggerAppenderFile',
              'layout' => array(
                'class' => 'LoggerLayoutSimple'
              ),
              'params' => array(
                  //'locationInfo' => true,
                  //'htmlLineBreaks' => 'true',
                  //'conversionPattern' => '%c - %d{Y-m-d H:i:s} %p - %m%n',
                  //'conversionPattern' => '%d{Y-m-d H:i:s} %p - %m%n',
                  'file' => $filepath,
                  'append' => true,
                ),
            )
          )
        )
      );
      $logger = \Logger::getLogger('default');
    }
     else {
      $logger = \Logger::getLogger(__CLASS__);
    }
    if ("" != $debugLevel) {
      $logger->setLevel($debugLevel);
    }
    self::setupDrupalBootstrap();

    switch ($mimetype) {
      case 'application/rdf+xml':
      case 'application/n3':
        $logger->trace('Creating a new XmlRdfAdapter');
        return new XmlRdfAdapter($debugLevel);
      case 'application/drupal':
      case 'drupal':
        $logger->trace('Creating a new DrupalAdapter');
        return new DrupalAdapter();
      case 'drupalx':
        $logger->trace('Creating a new DrupalxAdapter');
        return new DrupalxAdapter($debugLevel);
      default:
        throw new \UnexpectedValueException(
          "The input file mime type ({$mimetype}) is not managed.");
    }
  }

// end of member function getLoader

  /**
   * Setup drupal's api bootstrap
   * @throws \Exception
   */
  public static function setupDrupalBootstrap() {
    $current_dir = getcwd();

    // Set up of the drupal regarding the current configuration
    // Check that the variable is correctly configured
    if (!defined('DRUPAL_DIR')) {
      throw new \Exception('Drupal instance (DRUPAL_DIR) not configured');
    }

    // Check that the path exists
    if (!is_dir(DRUPAL_DIR)) {
      throw new \Exception('Drupal instance directory (DRUPAL_DIR) incorrect');
    }

    //required by Drupal
    chdir(DRUPAL_DIR);

    //load of the configuration file
    if (!function_exists('drupal_bootstrap')) {
      include_once('./includes/bootstrap.inc');
      global $_SERVER;
      $_SERVER['REMOTE_ADDR'] = 'localhost';
      //to avoid error messages without border effect
      $_SERVER['REQUEST_METHOD'] = 'inline';
      drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
    }
    chdir($current_dir);
  }
}

// end of LoaderFactory
?>
