<?php

namespace Adapters;

class DrupalxAdapter implements IExporter, ILoader {
  public $classes = array(
    'repository' => 'ADMSx\RdfClasses\AssetRepository',
    'project' => 'ADMSx\RdfClasses\SoftwareProject',
  );
  public $adms_object;
  protected $data;
  protected $procesedUris;
  private $loadedNids;
  public $node;

  function __construct() {
    $this->setupLogger();
    //not a good idea to use an array to store objects
  }
  
  private function createTemplateObject ($type) {
    switch ($type) {
      case 'repository' :
        return new \ADMSx\RdfClasses\AssetRepository();
      case 'project' :
        return new \ADMSx\RdfClasses\SoftwareProject();
      case 'contact_point' : 
        return new \ADMSx\RdfClasses\ContactInformation();
      case 'licence' : 
        return new \ADMSx\RdfClasses\Licence();
      case 'asset_release' : 
        return new \ADMSx\RdfClasses\Asset();
      case 'distribution' : 
        return new \ADMSx\RdfClasses\AssetDistribution();
      case 'checksum' : 
        return new \ADMSx\RdfClasses\Checksum();
      case 'data_set' :
        return new \ADMSx\RdfClasses\Dataset();
      case 'documentation' : 
        return new \ADMSx\RdfClasses\Documentation();
      case 'identifier' : 
        return new \ADMSx\RdfClasses\Identifier();
      case 'periodOfTime' :
        return new \ADMSx\RdfClasses\PeriodOfTime();
      case 'software_identifier' : 
        return new \ADMSx\RdfClasses\SoftwareIdentifier();
      case 'software_project_agent' : 
        return new \ADMSx\RdfClasses\SoftwareProjectAgent();
      case 'publisher' : 
        return new \ADMSx\RdfClasses\Publisher();
    }
    return false;
  }

  /**
   * Setup logger
   */
  private function setupLogger() {
    $this->_logger = \Logger::getLogger(__CLASS__);
  }

  /**
   * Import ADMS objects to Drupal
   *
   * @param $data
   * @param $output
   * @return bool|void
   */
  public function export($data, &$output,$import=true) {
    $this->data = $data;
    $this->output = $output;
    foreach ($this->data as &$value) {
      foreach ($value as &$object) {
        if (!empty($object)&&!in_array($object->id,$this->procesedUris)) {
          $this->procesedUris[] = $object->id;
          if($object->content_type!='periodOfTime'){
           /** @var $object \Common\ADMSObject */
           $object->importToDrupal($this->data, $this->output,$import);
          }
        }
      }
    }
    if($import){
      $this->procesedUris = array();
      foreach ($this->data as &$value) {
        foreach ($value as &$object) {
          if (!empty($object) && !in_array($object->id, $this->procesedUris)) {
            $this->procesedUris[] = $object->id;
            /** @var $object \Common\ADMSObject */
            $object->configureOgAncestry();
          }
        }
      }
    }
    return 'End of the drupal import';
  }

  public function load($nid) {
    $this->node = $this->getRepositories($nid);

    return $this->adms_object;
  }

  private function convertToADMSObject($nid) {
    $node = node_load($nid);
    if (is_object($node)) {
      $rdfObject = $this->createTemplateObject($node->type);
      if (is_object($rdfObject)) {
        $rdfObject->toADMSObject($node);
        if ($rdfObject) {
          $this->adms_object[$rdfObject->id][] = $rdfObject;
        }
      } else {
        watchdog("ADMS", "ADMSx definition for type " . $node->type . " doens't exist. Please check DrupalxAdapter", 
          array(), WATCHDOG_WARNING);
      }
    } else {
      drupal_set_message("Reference with id {$nid} doesn't exist. Please check your content for broken references.", "warning");
    }

    return $node;
  }

  private function getNodes($node, $types) {
    $node = $this->getReferencedNode($node->nid, true);
    $node->repository_tree = $this->getRepositoryNodes($node->nid, $types);
    return $node;
  }

  private function getRepositoryNodes($nid, $node_types) {
    $sql = sprintf('SELECT oga.nid FROM {og_ancestry} oga INNER JOIN {node} n ON oga.nid = n.nid WHERE oga.group_nid = %d AND n.type IN ("%s")', $nid, implode('","', $node_types));
    $result = db_query($sql);
    while ($row = db_fetch_array($result)) {
      $group_nodes[] = $row['nid'];
    }
    $this->addProjectRelease($nid, $group_nodes);

    return $group_nodes;
  }

  private function addProjectRelease($nid, &$group_nodes) {
    //$node = $this->convertToADMSObject($nid);
    $node = node_load($nid);
    if ('project_project' == $node->type and 'software' == $node->group_type) {
      $sql = "SELECT
      node.nid
      FROM node node
      LEFT JOIN project_release_nodes project_release_nodes ON node.nid = project_release_nodes.nid
      LEFT JOIN node node_project_release_nodes ON project_release_nodes.pid = node_project_release_nodes.nid
      INNER JOIN project_projects node_project_release_nodes__project_projects ON node_project_release_nodes.nid = node_project_release_nodes__project_projects.nid
      WHERE (node.type in ('project_release'))
      AND (node_project_release_nodes__project_projects.uri = '%s')";
      $result = db_query($sql, $node->project['uri']);
      while ($row = db_fetch_object($result)) {
        $group_nodes[] = $row->nid;
      }
    }
  }

  public function getReferencedNode($nid, $obtain_node = false) {
    if(!isset($this->loadedNids[$nid])){
      $this->loadedNids[$nid] = $nid;
      $node = $this->convertToADMSObject($nid);
      $info = _content_type_info();
      $fields = $info['content types'][$node->type]['fields'];
      $fields = $this->nodeReferenceTypes($fields);
      foreach($fields as $field){
        if(count($node->{$field}) > 0){
          foreach ($node->{$field} as $sub_value) {
            if ($sub_value['nid'] > 0) {
              $this->loadedNids[$sub_value['nid']] = $this->getReferencedNode($sub_value['nid']);
            }
          }
        }
      }
    }

    if($obtain_node) {
      return $node;
    }
    return $this->loadedNids[$nid];
  }

  private function nodeReferenceTypes($fields) {
    $nodeReference = array();
    foreach ($fields as $key => $value) {
      if ("nodereference" == $value['type']) {
        $nodeReference[] = $key;
      }
    }
    return $nodeReference;
  }

  private function getRepositories($nid) {
    //https://webgate.ec.europa.eu/CITnet/jira/browse/ISAICP-1057
    //Export pproject data
    $container = node_load($nid);
    if ('project_project' == $container->type) {
      if (0 == $container->field_project_common_type[0]['value']) {//asset project
         $sql = "SELECT node.nid
          FROM   node,og_ancestry
          WHERE  type = 'asset_release'
          AND og_ancestry.group_nid = %d
          AND og_ancestry.nid=node.nid;";
        $result = db_query($sql, $nid);
        while ($row = db_fetch_object($result)) {
          $node[] = $this->getNodes(node_load($row->nid), array("project", 'asset_release'));
        }
      }
      if (1 == $container->field_project_common_type[0]['value']) {//software project
        $node[] = $this->getNodes(node_load($container->field_project_adms_sw_project[0]['nid']), array("project", 'asset_release'));
      }
      return $node;
    }
    else {
      $result = db_query("SELECT nid FROM node WHERE type IN('repository', 'asset_release') AND nid=%d", $nid);
      while ($row = db_fetch_object($result)) {
        $node = $this->getNodes(node_load($row->nid), array("project", 'asset_release'));
      }
    }
    return $node;
  }
  /**
  * Changes the bootstrap to the Drupal site during cronjob execution
  */
  function setBootstrap() {
    $this->current_dir = getcwd();
    // Set up of the drupal regarding the current configuration
    // Check that the variable is correctly configured
    if (!defined('DRUPAL_DIR')) {
      throw new \Exception('Drupal instance (DRUPAL_DIR) not configured');
    }

    // Check that the path exists
    if (!is_dir(DRUPAL_DIR)) {
      throw new \Exception('Drupal instance directory (DRUPAL_DIR) incorrect');
    }

    //required by Drupal
    chdir(DRUPAL_DIR);

    //load of the configuration file
    if (!function_exists('drupal_bootstrap')) {
     include_once('./includes/bootstrap.inc');
      global $_SERVER;
      $_SERVER['REMOTE_ADDR'] = 'localhost';
      //to avoid error messages without border effect
      $_SERVER['REQUEST_METHOD'] = 'inline';
      drupal_bootstrap(DRUPAL_BOOTSTRAP_FULL);
    }
  }
  /**
  * Changes the bootstrap to the cronjob site during cronjob execution
  */
  function returnBoostrap() {
    chdir($this->current_dir);
  }
}
