<?php
/**
 * Rss2Rdf: A class to convert RSS to RDF in PHP
 * It converts a RSS file to RDF. To do that it applies a XSLT file to the RSS file and create a new RDF file
 *
 * Usage:
 *      Rss2Rdf::createRDF($file_rss, $path);
 */
class Rss2Rdf {
	
  private static $xsl_file = 'rss2rdf.xsl';

  /**
   * Convert an RSS file to RDF file
   * @param $input_rss - RSS filename
   * @param $path - RSS filename path
  */
  public static function createRDF($input_rss, $path) {
  	# LOAD RSS FILE 
	$xml = new DOMDocument();
	$rss_file = $path . '/' . $input_rss;
	$parsed_xml = $xml->load($rss_file); 
	if(!$parsed_xml) {
	  throw new Exception('[rss2rdf] Error parsing the RSS file.');
	}
	else {
	  # START XSLT 
	  $xslt = new XSLTProcessor(); 
	  $xsl = new DOMDocument(); 
	  $parsed_xsl = $xsl->load	(dirname(__FILE__) . '/' .  self::$xsl_file, LIBXML_NOCDATA); 
	  if(!$parsed_xsl) {
	  	throw new Exception('[rss2rdf] Error loading the XSL file.');
	  }
	  else {
	 	$res_xslt = $xslt->importStylesheet($xsl); 
		if(!$res_xslt) {
	      throw new Exception('[rss2rdf] Error applying the XSLT to the RSS file.');
		}
		else{
		  # GENERATE RDF
		  $output = $xslt->transformToXML($xml);
		  // Remove breaklines form the RDF code
		  $output = str_replace(array('&#13;','&#10;'), '', $output);
		  // Replace .rss with .rdf (filename.rdf)
		  $rdf_file = str_replace('.rss', '.rdf', $input_rss);
		  $path_rdf = $path . '/' . $rdf_file;
		  // Create the .rdf file
		  $f = fopen($path_rdf, 'w');
		  fwrite($f, $output);  
		  fclose($f); 
		}
	  }
	}
  }
}
